import { __awaiter } from 'tslib';
import { Injectable, NgZone, Optional, Component, ViewEncapsulation, Input, ChangeDetectorRef, forwardRef, EventEmitter, Output, Directive, ElementRef, Renderer2, HostListener, ChangeDetectionStrategy, NgModule, TemplateRef, ContentChildren, ViewChild, ViewContainerRef } from '@angular/core';
import Swagger from 'swagger-client';
import { HttpClient, HttpHeaders, HttpRequest, HttpEventType, HttpErrorResponse, HttpClientModule } from '@angular/common/http';
import { BehaviorSubject, Subject, throwError, Observable, ReplaySubject, combineLatest } from 'rxjs';
import { map, takeUntil, filter, debounceTime } from 'rxjs/operators';
import { CommonModule } from '@angular/common';
import { FormControl, FormGroup, Validators, FormArray, NG_VALUE_ACCESSOR, FormsModule, ReactiveFormsModule } from '@angular/forms';
import { NavigationEnd, Router, ActivatedRoute, RouterModule } from '@angular/router';
import { MarkdownModule } from 'ngx-markdown';
import { CdkPortal, PortalModule } from '@angular/cdk/portal';
import { Overlay, OverlayModule } from '@angular/cdk/overlay';
import { AceModule } from 'ngx-ace-wrapper';
import 'brace';
import 'brace/mode/json';
import 'brace/mode/xml';
import * as Ajv from 'ajv';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const httpMethods = ['get', 'put', 'post', 'delete', 'options', 'head', 'trace'];
/** @type {?} */
const securitySchemeTypes = ['apiKey', 'http', 'oauth2', 'openIdConnect'];
/** @type {?} */
const securitySchemeTypesWithScopes = ['oauth2', 'openIdConnect'];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const formatExampleDate = new Date();
formatExampleDate.setMilliseconds(0);
formatExampleDate.setSeconds(0);
const ɵ0 = /**
 * @return {?}
 */
() => '', ɵ1 = /**
 * @return {?}
 */
() => 'user@example.com', ɵ2 = /**
 * @return {?}
 */
() => formatExampleDate.toISOString(), ɵ3 = /**
 * @return {?}
 */
() => formatExampleDate.toISOString().substring(0, 10), ɵ4 = /**
 * @return {?}
 */
() => formatExampleDate.toISOString().substring(11), ɵ5 = /**
 * @return {?}
 */
() => '3fa85f64-5717-4562-b3fc-2c963f66afa6', ɵ6 = /**
 * @return {?}
 */
() => 'example.com', ɵ7 = /**
 * @return {?}
 */
() => '198.51.100.42', ɵ8 = /**
 * @return {?}
 */
() => '2001:0db8:5b96:0000:0000:426f:8e17:642a', ɵ9 = /**
 * @return {?}
 */
() => 0, ɵ10 = /**
 * @return {?}
 */
() => 0.0, ɵ11 = /**
 * @return {?}
 */
() => 0, ɵ12 = /**
 * @param {?} schema
 * @return {?}
 */
schema => (typeof schema.default === 'boolean' ? schema.default : true), ɵ13 = /**
 * @return {?}
 */
() => null, ɵ14 = /**
 * @return {?}
 */
() => null;
/** @type {?} */
const formatExamples = {
    string: (ɵ0),
    string_email: (ɵ1),
    'string_date-time': (ɵ2),
    string_date: (ɵ3),
    string_time: (ɵ4),
    string_uuid: (ɵ5),
    string_hostname: (ɵ6),
    string_ipv4: (ɵ7),
    string_ipv6: (ɵ8),
    number: (ɵ9),
    number_float: (ɵ10),
    integer: (ɵ11),
    boolean: (ɵ12),
    null: (ɵ13),
    any: (ɵ14)
};
/**
 * @param {?} schema
 * @return {?}
 */
function examplePrimitive(schema) {
    /** @type {?} */
    const type = Array.isArray(schema.type) ? schema.type[0] : schema.type;
    /** @type {?} */
    const formatFunc = formatExamples[`${type}_${schema.format}`] || formatExamples[type];
    if (typeof formatFunc === 'function') {
        return formatFunc(schema);
    }
    return `Unknown: ` + type;
}
/**
 * @param {?} schema
 * @return {?}
 */
function exampleFromSchema(schema) {
    /** @type {?} */
    const type = Array.isArray(schema.type) ? schema.type[0] : schema.type;
    if ('const' in schema) {
        return schema.const;
    }
    if ('default' in schema) {
        return schema.default;
    }
    if ('examples' in schema && schema.examples.length) {
        return schema.examples[0];
    }
    if (schema.enum) {
        return schema.enum[0];
    }
    if (type === 'object' || schema.properties) {
        /** @type {?} */
        const result = {};
        if (((/** @type {?} */ (schema))).example) {
            return ((/** @type {?} */ (schema))).example;
        }
        if (!schema.properties) {
            return result;
        }
        /** @type {?} */
        const keys = Object.keys(schema.properties);
        for (const key of keys) {
            result[key] = exampleFromSchema(schema.properties[key]);
        }
        return result;
    }
    if (type === 'array') {
        /** @type {?} */
        const item = Array.isArray(schema.items) ? schema.items[0] : schema.items;
        return [exampleFromSchema(item)];
    }
    return examplePrimitive(schema);
}
/**
 * @param {?} len
 * @return {?}
 */
function randomHex(len) {
    /** @type {?} */
    const preHex = len - 1 === 0 ? 0 : Math.pow(2, (4 * (len - 1)));
    /** @type {?} */
    const hex = Math.pow(2, (4 * len)) - 1;
    return Math.floor(preHex + Math.random() * (hex - preHex)).toString(16);
}
/**
 * @param {?} pattern
 * @return {?}
 */
function getStarMatcher(pattern) {
    /** @type {?} */
    const escaped = escapeRegExpNoStar(pattern);
    return new RegExp('^' + escaped.replace(/\*/g, '.{0,50}') + '$');
}
/**
 * @param {?} pattern
 * @return {?}
 */
function escapeRegExpNoStar(pattern) {
    return pattern.replace(/[.+?^${}()|[\]\\]/g, '\\$&');
}
/**
 * @param {?} input
 * @return {?}
 */
function getJsonString(input) {
    if (typeof input === 'string' && input.match(/^({|\[|"|\d|-\d|null)/)) {
        return input;
    }
    return JSON.stringify(input, null, 2);
}
/**
 * @param {?} queryParams
 * @return {?}
 */
function serializeQueryParams(queryParams) {
    if (!queryParams) {
        return '';
    }
    return Object.keys(queryParams)
        .map((/**
     * @param {?} key
     * @return {?}
     */
    key => encodeURIComponent(key) + '=' + encodeURIComponent(queryParams[key])))
        .join('&');
}
/**
 * @param {?} queryParams
 * @return {?}
 */
function parseQueryString(queryParams) {
    if (typeof queryParams !== 'string' || !queryParams.length) {
        return {};
    }
    /** @type {?} */
    const paramsList = queryParams.split('&');
    /** @type {?} */
    const params = {};
    if (paramsList.length) {
        for (const line of paramsList) {
            const [key, value] = line.split('=');
            params[decodeURIComponent(key)] = decodeURIComponent(value);
        }
    }
    return params;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class OpenapiAuthService {
    /**
     * @param {?} zone
     * @param {?} http
     */
    constructor(zone, http) {
        this.zone = zone;
        this.http = http;
        this.securitySchemes = new BehaviorSubject([]);
        /**
         * Emits with the scheme name when its credentials have been updated
         */
        this.updatedCredentials = new Subject();
        this.globalRequirements = [];
        this.currentWindow = window;
    }
    /**
     * @param {?} spec
     * @return {?}
     */
    identifySchemes(spec) {
        this.securitySchemes.next(getSecurityInformation(spec));
        if (Array.isArray(spec.security)) {
            this.globalRequirements = spec.security;
        }
    }
    /**
     * @param {?=} requirement
     * @return {?}
     */
    getAllRequirements(requirement) {
        return [...this.globalRequirements, ...(requirement || [])].filter((/**
         * @param {?} reqObject
         * @return {?}
         */
        reqObject => Object.keys(reqObject).length > 0));
    }
    /**
     * @param {?=} requirement
     * @return {?}
     */
    getRequiredSchemes(requirement) {
        /** @type {?} */
        const result = [];
        for (const reqs of this.getAllRequirements(requirement)) {
            result.push(Object.keys(reqs).map((/**
             * @param {?} reqName
             * @return {?}
             */
            reqName => {
                /** @type {?} */
                const schemaItem = this.getSchema(reqName);
                return {
                    name: reqName,
                    scopes: reqs[reqName],
                    securityScheme: schemaItem.securityScheme,
                    authenticated: schemaItem.authenticated
                };
            })));
        }
        return result;
    }
    /**
     * @param {?} requirements
     * @return {?}
     */
    getAuthStatus(requirements) {
        /** @type {?} */
        let authStatus = 'none';
        /** @type {?} */
        const allRequirements = this.getAllRequirements(requirements);
        if (allRequirements && Object.keys(allRequirements).length) {
            authStatus = 'required';
            if (this.hasAnyAuthorization(allRequirements)) {
                authStatus = 'ok';
            }
        }
        return authStatus;
    }
    /**
     * @param {?} requirements
     * @return {?}
     */
    hasAnyAuthorization(requirements) {
        if (!requirements || !requirements.length) {
            return true;
        }
        // one needs to be fulfilled
        for (const req of requirements) {
            /** @type {?} */
            const reqNames = Object.keys(req);
            if (reqNames.length && reqNames.every((/**
             * @param {?} rn
             * @return {?}
             */
            rn => this.hasRequirement(rn, req[rn])))) {
                return true;
            }
        }
        return false;
    }
    /**
     * @param {?} reqName
     * @param {?} scopes
     * @return {?}
     */
    hasRequirement(reqName, scopes) {
        /** @type {?} */
        const scheme = this.getSchema(reqName);
        if (scheme &&
            scheme.authenticated &&
            scheme.securityScheme.type &&
            securitySchemeTypesWithScopes.includes(scheme.securityScheme.type)) {
            /** @type {?} */
            const credentials = (/** @type {?} */ (scheme.credentials));
            if (credentials && credentials.scopes && scopes.every((/**
             * @param {?} s
             * @return {?}
             */
            s => credentials.scopes.includes(s)))) {
                return true;
            }
        }
        else if (scheme && scheme.authenticated) {
            return true;
        }
        return false;
    }
    /**
     * @param {?} name
     * @return {?}
     */
    getSchema(name) {
        return this.securitySchemes.value.find((/**
         * @param {?} s
         * @return {?}
         */
        s => s.name === name));
    }
    /**
     * @param {?} name
     * @param {?} credentials
     * @param {?} authenticated
     * @param {?=} remember
     * @return {?}
     */
    updateCredentials(name, credentials, authenticated, remember = false) {
        if (typeof name !== 'string' || !name.length) {
            throw new Error('No credentials name given');
        }
        /** @type {?} */
        const schema = this.getSchema(name);
        schema.authenticated = authenticated;
        if (schema.authenticated) {
            schema.credentials = credentials;
            if (remember) {
                schema.remember = true;
                writeCredentialsStore(name, credentials);
            }
        }
        else {
            schema.remember = false;
            schema.credentials = null;
            clearCredentialsStore(name);
        }
        this.updatedCredentials.next(name);
    }
    /**
     * @param {?} name
     * @param {?} credentials
     * @return {?}
     */
    runOAuthAuthorization(name, credentials) {
        /** @type {?} */
        const scheme = this.getSchema(name);
        if (!scheme || !credentials) {
            return throwError(new Error('Unknown scheme or no credentials given'));
        }
        scheme.credentials = credentials;
        scheme.remember = true;
        writeCredentialsStore(name, credentials);
        if (credentials.flow === 'implicit' || credentials.flow === 'authorizationCode') {
            return new Observable((/**
             * @param {?} subscriber
             * @return {?}
             */
            subscriber => {
                /** @type {?} */
                const url = buildAuthorizationUrl(scheme.securityScheme, credentials, name);
                /** @type {?} */
                const service = this;
                ((/** @type {?} */ (this.currentWindow))).handleOAuthCallback = (/**
                 * @param {?} fragment
                 * @return {?}
                 */
                fragment => {
                    // called from other window
                    return service.zone.run((/**
                     * @return {?}
                     */
                    () => service.handleOAuthCallback(fragment).then((/**
                     * @return {?}
                     */
                    () => {
                        subscriber.next(true);
                        subscriber.complete();
                    }), (/**
                     * @param {?} err
                     * @return {?}
                     */
                    err => {
                        subscriber.error(err);
                    }))));
                });
                this.currentWindow.open(url, '_blank');
            }));
        }
        if (credentials.flow === 'clientCredentials') {
            /** @type {?} */
            const tokenUrl = getTokenBaseUrl(scheme.securityScheme, 'clientCredentials');
            return this.requestToken(tokenUrl, 'client_credentials', credentials).pipe(map((/**
             * @param {?} token
             * @return {?}
             */
            token => {
                credentials.token = token;
                this.updateCredentials(name, credentials, true, true);
                return true;
            })));
        }
        if (credentials.flow === 'password') {
            /** @type {?} */
            const tokenUrl = getTokenBaseUrl(scheme.securityScheme, 'password');
            return this.requestToken(tokenUrl, 'password', credentials).pipe(map((/**
             * @param {?} token
             * @return {?}
             */
            token => {
                credentials.token = token;
                this.updateCredentials(name, credentials, true, true);
                return true;
            })));
        }
        return throwError(new Error('Unknown flow'));
    }
    /**
     * @param {?} params
     * eg. access_token=8c665834-6a43-4704-81d3-42d311a1ab91&token_type=bearer&expires_in=20&scope=read%20write:pets%20write%20read:pets
     * @return {?}
     */
    handleOAuthCallback(params) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.currentWindow.opener && this.currentWindow.opener.handleOAuthCallback) {
                yield this.currentWindow.opener.handleOAuthCallback(params);
                this.currentWindow.close();
                return;
            }
            if (!params.state) {
                return;
            }
            /** @type {?} */
            const scheme = this.getSchema(params.state);
            if (!scheme || !scheme.credentials) {
                return;
            }
            /** @type {?} */
            const credentials = (/** @type {?} */ (scheme.credentials));
            if (params.access_token) {
                return this.updateByImplicitFlow(scheme.name, credentials, params);
            }
            if (params.code) {
                return yield this.updateByCodeFlow(scheme, credentials, params);
            }
        });
    }
    /**
     * @private
     * @param {?} name
     * @param {?} credentials
     * @param {?} params
     * @return {?}
     */
    updateByImplicitFlow(name, credentials, params) {
        credentials.token = {
            access_token: params.access_token,
            token_type: params.token_type
        };
        if (params.expires_in) {
            credentials.token.expires_at = new Date().getTime() + parseInt(params.expires_in, 10) * 1000;
        }
        this.updateCredentials(name, credentials, true, true);
    }
    /**
     * @private
     * @param {?} scheme
     * @param {?} credentials
     * @param {?} params
     * @return {?}
     */
    updateByCodeFlow(scheme, credentials, params) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            const tokenUrl = getTokenBaseUrl(scheme.securityScheme, 'authorizationCode');
            credentials.token = yield this.requestToken(tokenUrl, 'authorization_code', credentials, params.code).toPromise();
            this.updateCredentials(name, credentials, true, true);
        });
    }
    /**
     * Updates the provided credentials with a token
     * @private
     * @param {?} tokenUrl
     * @param {?} grantType
     * @param {?} credentials
     * @param {?=} code
     * @return {?}
     */
    requestToken(tokenUrl, grantType, credentials, code) {
        /** @type {?} */
        const tokenRequest = {
            grant_type: grantType,
            client_id: credentials.clientId,
            client_secret: credentials.clientSecret
        };
        if (grantType === 'authorization_code') {
            tokenRequest.code = code;
            tokenRequest.redirect_uri = credentials.redirectUri;
        }
        if (grantType === 'password') {
            tokenRequest.username = credentials.username;
            tokenRequest.password = credentials.password;
        }
        if (!this.http) {
            throw new Error('No http client provider available.');
        }
        return this.http
            .post(tokenUrl, serializeQueryParams(tokenRequest), {
            headers: {
                'Content-type': 'application/x-www-form-urlencoded'
            }
        })
            .pipe(map((/**
         * @param {?} result
         * @return {?}
         */
        result => {
            if (result && result.access_token) {
                /** @type {?} */
                const token = {
                    access_token: result.access_token,
                    token_type: result.token_type,
                    id_token: result.id_token,
                    refresh_token: result.refresh_token
                };
                if (result.expires_in) {
                    token.expires_at = new Date().getTime() + parseInt(result.expires_in, 10) * 1000;
                }
                return token;
            }
            else {
                throw new Error('Missing access token in response from token endpoint ' + tokenUrl);
            }
        })));
    }
    /**
     * @param {?} name
     * @return {?}
     */
    removeToken(name) {
        /** @type {?} */
        const schema = this.getSchema(name);
        if (!schema || !schema.credentials) {
            return;
        }
        ((/** @type {?} */ (schema.credentials))).token = null;
        schema.authenticated = false;
        writeCredentialsStore(name, schema.credentials);
        this.updatedCredentials.next(name);
    }
}
OpenapiAuthService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
OpenapiAuthService.ctorParameters = () => [
    { type: NgZone },
    { type: HttpClient, decorators: [{ type: Optional }] }
];
if (false) {
    /** @type {?} */
    OpenapiAuthService.prototype.securitySchemes;
    /**
     * Emits with the scheme name when its credentials have been updated
     * @type {?}
     */
    OpenapiAuthService.prototype.updatedCredentials;
    /**
     * @type {?}
     * @private
     */
    OpenapiAuthService.prototype.globalRequirements;
    /** @type {?} */
    OpenapiAuthService.prototype.currentWindow;
    /**
     * @type {?}
     * @private
     */
    OpenapiAuthService.prototype.zone;
    /**
     * @type {?}
     * @private
     */
    OpenapiAuthService.prototype.http;
}
/**
 * @param {?} name
 * @return {?}
 */
function clearCredentialsStore(name) {
    /** @type {?} */
    const key = getStorageKey(name);
    window.sessionStorage.removeItem(key);
}
/**
 * @param {?} name
 * @param {?} credentials
 * @return {?}
 */
function writeCredentialsStore(name, credentials) {
    /** @type {?} */
    const key = getStorageKey(name);
    /** @type {?} */
    const value = JSON.stringify(credentials);
    window.sessionStorage.setItem(key, value);
}
/**
 * @param {?} name
 * @return {?}
 */
function readCredentialsStore(name) {
    /** @type {?} */
    const key = getStorageKey(name);
    /** @type {?} */
    const value = window.sessionStorage.getItem(key);
    if (typeof value === 'string') {
        try {
            return JSON.parse(value);
        }
        catch (e) {
            console.warn('Failed to parse credentials for ' + name, e);
        }
    }
    return null;
}
/**
 * @param {?} name
 * @return {?}
 */
function getStorageKey(name) {
    return 'oavCredentials.' + name.replace(/[^-_A-Za-z0-9]/g, '_');
}
/**
 * @param {?} spec
 * @return {?}
 */
function getSecurityInformation(spec) {
    /** @type {?} */
    const securitySchemes = [];
    /** @type {?} */
    const definitions = spec.securityDefinitions || (spec.components && spec.components.securitySchemes) || {};
    for (const [schemeName, schemeDef] of Object.entries(definitions)) {
        /** @type {?} */
        const storedCredentials = readCredentialsStore(schemeName);
        securitySchemes.push({
            name: schemeName,
            securityScheme: (/** @type {?} */ (schemeDef)),
            authenticated: storedCredentials !== null,
            remember: storedCredentials !== null,
            credentials: storedCredentials
        });
    }
    return securitySchemes;
}
/**
 * @param {?} scheme
 * @return {?}
 */
function identifyFlows(scheme) {
    /** @type {?} */
    const results = [];
    // V2
    if (scheme.flow) {
        /** @type {?} */
        let flowName = scheme.flow;
        if (flowName === 'application') {
            flowName = 'clientCredentials';
        }
        if (flowName === 'accessCode') {
            flowName = 'authorizationCode';
        }
        results.push({
            flow: flowName,
            scopes: toScopesInfo(scheme.scopes)
        });
    }
    if (scheme.flows) {
        if (scheme.flows.implicit) {
            results.push({
                flow: 'implicit',
                scopes: toScopesInfo(scheme.flows.implicit.scopes)
            });
        }
        if (scheme.flows.password) {
            results.push({
                flow: 'password',
                scopes: toScopesInfo(scheme.flows.password.scopes)
            });
        }
        if (scheme.flows.clientCredentials) {
            results.push({
                flow: 'clientCredentials',
                scopes: toScopesInfo(scheme.flows.clientCredentials.scopes)
            });
        }
        if (scheme.flows.authorizationCode) {
            results.push({
                flow: 'authorizationCode',
                scopes: toScopesInfo(scheme.flows.authorizationCode.scopes)
            });
        }
    }
    return results;
}
/**
 * @param {?} scopes
 * @return {?}
 */
function toScopesInfo(scopes) {
    if (!scopes) {
        return [];
    }
    return Object.keys(scopes).map((/**
     * @param {?} name
     * @return {?}
     */
    name => ({ scope: name, description: scopes[name] })));
}
/**
 * @param {?} scheme
 * @param {?} flow
 * @return {?}
 */
function getAuthorizationBaseUrl(scheme, flow) {
    if (scheme.type === 'oauth2' && (scheme.flow === 'implicit' || scheme.flow === 'accessCode')) {
        return scheme.authorizationUrl;
    }
    if (scheme.type === 'oauth2' && scheme.flows && scheme.flows[flow]) {
        return scheme.flows[flow].authorizationUrl;
    }
    return null;
}
/**
 * @param {?} scheme
 * @param {?} flow
 * @return {?}
 */
function getTokenBaseUrl(scheme, flow) {
    if (scheme.type === 'oauth2' && scheme.flow === 'accessCode') {
        return scheme.tokenUrl;
    }
    if (scheme.type === 'oauth2' && scheme.flows && scheme.flows[flow]) {
        return scheme.flows[flow].tokenUrl;
    }
    return null;
}
/**
 * @param {?} scheme
 * @param {?} credentials
 * @param {?=} state
 * @return {?}
 */
function buildAuthorizationUrl(scheme, credentials, state) {
    /** @type {?} */
    const baseUrl = getAuthorizationBaseUrl(scheme, credentials.flow);
    /** @type {?} */
    const params = {
        client_id: credentials.clientId,
        redirect_uri: credentials.redirectUri
    };
    if (credentials.flow === 'implicit') {
        params.response_type = 'token';
        params.nonce = credentials.nonce;
    }
    if (credentials.flow === 'authorizationCode') {
        params.response_type = 'code';
    }
    if (state) {
        params.state = state;
    }
    return baseUrl + '?' + serializeQueryParams(params);
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class OpenapiViewerService {
    /**
     * @param {?} http
     * @param {?} authService
     */
    constructor(http, authService) {
        this.http = http;
        this.authService = authService;
        this.spec = new BehaviorSubject(null);
        this.tagIndex = new BehaviorSubject([]);
        this.loadErrors = new BehaviorSubject([]);
        this.operationParameterCache = {};
        this.requests = [];
        this.loadingPromise = null;
    }
    /**
     * @param {?} spec
     * @return {?}
     */
    loadSpec(spec) {
        return this.resolveSpec({ spec });
    }
    /**
     * @param {?} url
     * @return {?}
     */
    loadSpecByUrl(url) {
        return this.resolveSpec({ url });
    }
    /**
     * @private
     * @param {?} swaggerOpts
     * @return {?}
     */
    resolveSpec(swaggerOpts) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.loadErrors.value.length) {
                this.loadErrors.next([]);
            }
            if (this.tagIndex.value.length) {
                this.tagIndex.next([]);
            }
            this.loadingPromise = Swagger.resolve({ spec: swaggerOpts.spec, url: swaggerOpts.url });
            /** @type {?} */
            const loadingPromise = this.loadingPromise;
            /** @type {?} */
            const resolveResult = yield loadingPromise;
            if (loadingPromise !== this.loadingPromise) {
                return null;
            }
            this.spec.next(resolveResult.spec);
            this.tagIndex.next(getTagIndex(resolveResult.spec));
            this.authService.identifySchemes(resolveResult.spec);
            this.operationParameterCache = {};
            if (resolveResult.errors && resolveResult.errors.length) {
                this.loadErrors.next(resolveResult.errors);
                throw new Error('Spec resolve errors');
            }
            return resolveResult.spec;
        });
    }
    /**
     * @param {?} operationId
     * @param {?} parameters
     * @param {?} requestBody
     * @param {?} requestContentType
     * @param {?} responseContentType
     * @return {?}
     */
    createRequest(operationId, parameters, requestBody, requestContentType, responseContentType) {
        // See https://github.com/swagger-api/swagger-js/blob/master/src/execute/index.js#L91
        // See https://github.com/swagger-api/swagger-js/blob/master/src/execute/index.js#L91
        /** @type {?} */
        const securities = {};
        for (const scheme of this.authService.securitySchemes.value) {
            if (scheme.authenticated) {
                securities[scheme.name] = scheme.credentials;
            }
        }
        /** @type {?} */
        const params = {
            spec: this.spec.value,
            operationId,
            parameters,
            securities: { authorized: securities },
            requestContentType,
            responseContentType,
            scheme: undefined,
            requestInterceptor: undefined,
            responseInterceptor: undefined,
            contextUrl: undefined,
            userFetch: undefined,
            requestBody,
            server: undefined,
            serverVariables: undefined,
            http: undefined
        };
        return Swagger.buildRequest(params);
    }
    /**
     * @param {?} pathItem
     * @param {?} operationsItem
     * @param {?} request
     * @return {?}
     */
    runRequest(pathItem, operationsItem, request) {
        /** @type {?} */
        const httpEvents = new ReplaySubject(1);
        /** @type {?} */
        const reqInfo = {
            id: randomHex(8),
            pathItem,
            operationsItem,
            request,
            running: true,
            canceled: false,
            sentBytes: 0,
            receivedBytes: 0,
            startTs: new Date(),
            endTs: null,
            httpEvents,
            statusText: null,
            status: 0,
            cancel: null
        };
        this.requests.push(reqInfo);
        /** @type {?} */
        const headers = new HttpHeaders(request.headers);
        /** @type {?} */
        const sub = this.http
            .request(new HttpRequest(request.method, request.url, request.body, {
            headers,
            responseType: 'blob',
            reportProgress: true
        }))
            .subscribe((/**
         * @param {?} status
         * @return {?}
         */
        status => {
            if (status.type === HttpEventType.ResponseHeader || status.type === HttpEventType.Response) {
                reqInfo.status = status.status;
                reqInfo.statusText = status.statusText;
            }
            if (status.type === HttpEventType.UploadProgress) {
                reqInfo.sentBytes = status.loaded;
            }
            if (status.type === HttpEventType.DownloadProgress) {
                reqInfo.receivedBytes = status.loaded;
            }
            if (status.type === HttpEventType.Response) {
                reqInfo.endTs = new Date();
                reqInfo.running = false;
            }
            console.log('status', status);
            httpEvents.next(status);
        }), (/**
         * @param {?} err
         * @return {?}
         */
        err => {
            reqInfo.endTs = new Date();
            reqInfo.running = false;
            httpEvents.error(err);
        }), (/**
         * @return {?}
         */
        () => {
            reqInfo.running = false;
            httpEvents.complete();
        }));
        reqInfo.cancel = (/**
         * @return {?}
         */
        () => {
            sub.unsubscribe();
            reqInfo.canceled = true;
            reqInfo.running = false;
        });
        return reqInfo;
    }
    /**
     * @param {?} req
     * @return {?}
     */
    removeRequest(req) {
        this.requests.splice(this.requests.indexOf(req), 1);
    }
    /**
     * @param {?} opId
     * @return {?}
     */
    getRequestsByOperationId(opId) {
        return this.requests.filter((/**
         * @param {?} req
         * @return {?}
         */
        req => req.operationsItem.operation.operationId === opId));
    }
    /**
     * @param {?} opId
     * @param {?} parameters
     * @return {?}
     */
    saveOperationParameters(opId, parameters) {
        this.operationParameterCache[opId] = parameters;
    }
    /**
     * @param {?} opId
     * @return {?}
     */
    loadOperationParameters(opId) {
        return this.operationParameterCache[opId];
    }
    /**
     * @return {?}
     */
    resetSpec() {
        this.spec.next(null);
        this.tagIndex.next([]);
        this.loadErrors.next([]);
        this.requests = [];
        this.operationParameterCache = {};
    }
}
OpenapiViewerService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
OpenapiViewerService.ctorParameters = () => [
    { type: HttpClient },
    { type: OpenapiAuthService }
];
if (false) {
    /** @type {?} */
    OpenapiViewerService.prototype.spec;
    /** @type {?} */
    OpenapiViewerService.prototype.tagIndex;
    /** @type {?} */
    OpenapiViewerService.prototype.loadErrors;
    /**
     * @type {?}
     * @private
     */
    OpenapiViewerService.prototype.operationParameterCache;
    /**
     * @type {?}
     * @private
     */
    OpenapiViewerService.prototype.requests;
    /**
     * @type {?}
     * @private
     */
    OpenapiViewerService.prototype.loadingPromise;
    /**
     * @type {?}
     * @private
     */
    OpenapiViewerService.prototype.http;
    /**
     * @type {?}
     * @private
     */
    OpenapiViewerService.prototype.authService;
}
/**
 * @param {?} spec
 * @return {?}
 */
function getTagIndex(spec) {
    return (spec.tags || [{ name: 'untagged' }]).map((/**
     * @param {?} tag
     * @return {?}
     */
    tag => ({
        tag,
        paths: getPathsByTag(spec, tag.name)
    })));
}
/**
 * @param {?} spec
 * @param {?} tag
 * @return {?}
 */
function getPathsByTag(spec, tag) {
    /** @type {?} */
    const paths = Object.keys(spec.paths);
    /** @type {?} */
    const result = [];
    for (const path of paths) {
        /** @type {?} */
        const pathObject = spec.paths[path];
        /** @type {?} */
        const tags = getTagsOfPath(pathObject);
        if ((tag !== 'untagged' && tags.includes(tag)) || (tag === 'untagged' && tags.length === 0)) {
            /** @type {?} */
            const pathItem = {
                path,
                summary: pathObject.summary,
                description: pathObject.description,
                parameters: (/** @type {?} */ (pathObject.parameters)),
                servers: pathObject.servers,
                operations: null
            };
            pathItem.operations = getOperationsOfPath(pathObject, pathItem.parameters);
            result.push(pathItem);
        }
    }
    return result;
}
/**
 * @param {?} pathObject
 * @param {?=} parentParameters
 * @return {?}
 */
function getOperationsOfPath(pathObject, parentParameters) {
    /** @type {?} */
    const ops = [];
    for (const method of httpMethods) {
        /** @type {?} */
        const operation = pathObject[method];
        if (operation) {
            if (!operation.operationId) {
                operation.operationId = 'op-' + randomHex(8);
            }
            /** @type {?} */
            const parameters = [];
            if (parentParameters) {
                parameters.push(...parentParameters);
            }
            if (operation.parameters) {
                parameters.push(...operation.parameters);
            }
            ops.push({
                method,
                operation,
                parameters,
                responseTypes: identifyResponseTypes(operation)
            });
        }
    }
    return ops;
}
/**
 * @param {?} pathObject
 * @return {?}
 */
function getTagsOfPath(pathObject) {
    /** @type {?} */
    const tags = [];
    for (const method of httpMethods) {
        /** @type {?} */
        const operation = pathObject[method];
        if (operation && operation.tags) {
            tags.push(...operation.tags);
        }
    }
    return tags;
}
/**
 * @param {?} operation
 * @return {?}
 */
function identifyResponseTypes(operation) {
    if (operation.produces) {
        return operation.produces;
    }
    /** @type {?} */
    const types = new Set();
    for (const [status, res] of Object.entries(operation.responses)) {
        if (res.content) {
            for (const contentType of Object.keys(res.content)) {
                types.add(contentType);
            }
        }
    }
    return [...types];
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class OpenapiViewerComponent {
    /**
     * @param {?} openApiService
     */
    constructor(openApiService) {
        this.openApiService = openApiService;
        this.index = [];
        this.loading = false;
        this.error = null;
        this.loadErrors = [];
        this.destroy = new Subject();
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.openApiService.tagIndex.pipe(takeUntil(this.destroy)).subscribe((/**
         * @param {?} tagIndex
         * @return {?}
         */
        tagIndex => {
            this.index = tagIndex;
        }));
        this.openApiService.loadErrors.pipe(takeUntil(this.destroy)).subscribe((/**
         * @param {?} errors
         * @return {?}
         */
        errors => {
            this.loadErrors = errors;
        }));
    }
    /**
     * @param {?} promise
     * @return {?}
     */
    loadSpec(promise) {
        this.loading = true;
        this.error = null;
        return promise.then((/**
         * @return {?}
         */
        () => {
            this.error = null;
            this.loading = false;
        }), (/**
         * @param {?} e
         * @return {?}
         */
        e => {
            this.loading = false;
            this.error = e.message;
            console.error(e);
        }));
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.destroy.complete();
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (changes.spec || changes.specUrl) {
            if (this.spec) {
                this.loadSpec(this.openApiService.loadSpec(this.spec));
            }
            else if (this.specUrl) {
                this.loadSpec(this.openApiService.loadSpecByUrl(this.specUrl));
            }
            else {
                this.openApiService.resetSpec();
            }
        }
    }
}
OpenapiViewerComponent.decorators = [
    { type: Component, args: [{
                selector: 'oav-openapi-viewer',
                template: "<div class=\"oav-layout\">\n  <header class=\"oav-header\">\n    <ng-content></ng-content>\n  </header>\n  <div class=\"oav-menu-header\">\n    <a href=\"javascript:\" class=\"oav-menu-toggle\" oavOpenable #openMenu=\"openable\"></a>\n  </div>\n  <div class=\"oav-content-container\">\n    <div *ngIf=\"loading\">\n      <span class=\"loading\"></span>\n      Loading\n    </div>\n\n    <div *ngIf=\"!loading && !index.length\">\n      No Tags found\n    </div>\n\n    <div *ngIf=\"!loading && error\">\n      <h3>Error</h3>\n      <pre>{{ error.toString() }}</pre>\n      <pre *ngFor=\"let err of loadErrors\">{{ err.toString() }}</pre>\n    </div>\n\n    <ng-container *ngIf=\"!loading && !error && index.length\">\n      <aside class=\"oav-index-container\" [class.open]=\"openMenu.open\">\n        <oav-index-nav [index]=\"index\"></oav-index-nav>\n      </aside>\n      <section class=\"oav-view-container\" [class.open]=\"openMenu.open\">\n        <router-outlet></router-outlet>\n      </section>\n    </ng-container>\n  </div>\n</div>\n",
                providers: [OpenapiAuthService, OpenapiViewerService],
                encapsulation: ViewEncapsulation.None,
                styles: ["@charset \"UTF-8\";.oav-layout{height:100vh;display:-webkit-box;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;flex-direction:column}.oav-layout .oav-menu-header{border-bottom:1px solid #ccc}.oav-layout .oav-menu-header .oav-menu-toggle{display:inline-block;color:#000;padding:.5rem;text-decoration:none;font-weight:700;font-size:1.2rem}.oav-layout .oav-menu-header .oav-menu-toggle::before{content:\"\u2630\"}.oav-layout .oav-content-container{-webkit-box-flex:1;flex:1;display:-webkit-box;display:flex;min-height:0;position:relative}.oav-layout .oav-content-container .oav-index-container{width:18rem;margin-left:-18rem;overflow:auto;background:#f6f6fb;-webkit-transition:margin-left .1s ease-out;transition:margin-left .1s ease-out}.oav-layout .oav-content-container .oav-index-container.open{margin-left:0}.oav-layout .oav-content-container .oav-view-container{-webkit-box-flex:1;flex:1;overflow:scroll;margin-right:0;-webkit-transition:margin-right .1s ease-out;transition:margin-right .1s ease-out}.oav-layout .oav-content-container .oav-view-container.open{margin-right:-18rem}.oav-content{margin:1rem}.oav-content h1{font-size:1.5em}.oav-content h3{font-size:1.15em}.oav-content h4{font-size:1em}.oav-content a{color:#2681ac}.oav-content a:hover{color:#31b1ff}.oav-content .deprecated{text-decoration:line-through}.oav-content .oav-path{font-family:monospace}.oav-content .oav-summary{font-weight:400}.oav-content .send-btn{color:#fff;background-color:#2681ac;font-size:1rem;padding:.5rem 1rem;border:0;margin:.2rem 0}.oav-content .send-btn:hover{background-color:#31b1ff}.oav-content .send-btn:disabled{background-color:#ccc}.oav-content pre{white-space:pre-wrap}.oav-content .oav-warning{color:#fb9200}.oav-content .ace_editor{border:1px solid #ccc}oav-json-schema .oav-json-schema-code{font-family:monospace}oav-json-schema .oav-json-schema-toggle{font-weight:700;text-decoration:none}oav-json-schema .json-schema-property{display:-webkit-box;display:flex;-webkit-box-align:start;align-items:flex-start}oav-json-schema .oav-json-schema-name.oav-json-schema-required::after{content:\"*\";color:#ac0003;padding-left:.1em}oav-json-schema .oav-json-schema-description{font-size:.9rem;margin-left:.5rem}oav-json-schema .oav-json-schema-description p{margin-top:0}oav-json-schema .oav-json-schema-description p:last-child{margin-bottom:0}oav-json-schema .oav-json-schema-type{margin-left:.2rem}oav-json-schema .oav-json-schema-type-name{color:#8a8a8f}.oav-method{width:5em;display:inline-block;color:#fff;text-align:center;padding:.2em;font-size:.7em}.oav-method.oav-method-get{background-color:#2681ac}.oav-method.oav-method-post{background-color:#25ac20}.oav-method.oav-method-put{background-color:#ac700e}.oav-method.oav-method-delete{background-color:#ac0003}.oav-method.oav-method-options{background-color:#ac03a6}.oav-method.oav-method-head{background-color:#87aaac}.oav-method.oav-method-trace{background-color:#87ac89}a.oav-header-link{color:#000;text-decoration:none}a.oav-header-link:hover{color:#2681ac}.oav-method-bg.method-get{background-color:rgba(38,129,172,.1)}.oav-method-bg.method-post{background-color:rgba(37,172,32,.1)}.oav-method-bg.method-put{background-color:rgba(172,112,14,.1)}.oav-method-bg.method-delete{background-color:rgba(172,0,3,.1)}.oav-method-bg.method-options{background-color:rgba(172,3,166,.1)}.oav-method-bg.method-head{background-color:rgba(135,170,172,.1)}.oav-method-bg.method-trace{background-color:rgba(135,172,137,.1)}oav-method-header h3{cursor:pointer;display:-webkit-box;display:flex;-webkit-box-align:center;align-items:center;padding:.5rem;margin:0}oav-method-header h3 .mainlink{text-decoration:inherit;color:inherit;cursor:pointer;font-size:1rem}oav-method-header h3 .method-get{background-color:#2681ac}oav-method-header h3 .method-post{background-color:#25ac20}oav-method-header h3 .method-put{background-color:#ac700e}oav-method-header h3 .method-delete{background-color:#ac0003}oav-method-header h3 .method-options{background-color:#ac03a6}oav-method-header h3 .method-head{background-color:#87aaac}oav-method-header h3 .method-trace{background-color:#87ac89}oav-method-header h3:hover{background-color:rgba(135,172,137,.2)}oav-method-header h3 .path{font-family:monospace;font-size:1.2rem}oav-method-header h3 .summary{padding:0 10px;font-size:.8rem;font-weight:400;-webkit-box-flex:1;flex:1}oav-method-header h3 .summary.extra-link{-webkit-box-flex:0;flex:none}.openapi-operation{margin:.5rem 0}.openapi-operation .oav-operation-details{display:none}.openapi-operation.open{padding-bottom:1rem}.openapi-operation.open .oav-operation-details{display:block}.openapi-operation.deprecated h3 .mainlink{text-decoration:line-through}.oav-operation-details{margin:.5rem 0;padding:0 2rem}.oav-operation-details .send-btn{color:#fff;background-color:#2681ac;font-size:1rem;padding:.5rem 1rem;border:0;margin:.2rem 0}.oav-operation-details .send-btn:hover{background-color:#31b1ff}.oav-operation-details pre{white-space:pre-wrap}.oav-operation-details h4{background-color:rgba(255,255,255,.6);padding:.5rem 2rem;margin:.5rem -2rem}.oav-operation-layout{display:-webkit-box;display:flex}.oav-operation-layout .oav-operation-details{-webkit-box-flex:1;flex:1}.oav-operation-layout .oav-operation-requests{min-width:30rem;margin-left:1rem}@media (max-width:80rem){.oav-operation-layout{display:block}.oav-operation-layout .oav-operation-requests{margin-left:0}}.toggle-button{border:none;background:0 0;outline:0}.toggle-button:before{font-size:1rem;font-weight:600;content:\"\u3009\";display:inline-block;-webkit-transform-origin:25% 50%;transform-origin:25% 50%;-webkit-transition:-webkit-transform .2s ease-out;transition:transform .2s ease-out;transition:transform .2s ease-out,-webkit-transform .2s ease-out}.toggle-button.open:before{-webkit-transform:rotate(90deg);transform:rotate(90deg)}.toggle-button.full-rotate.open:before{-webkit-transform:rotate(180deg);transform:rotate(180deg)}.download-btn{color:#fff;background-color:#2681ac;font-size:1rem;padding:.5rem 1rem;border:0;margin:.2rem 0}.download-btn:hover{background-color:#31b1ff}.dismiss-btn{color:#fff;background-color:#ac0003;font-size:1rem;padding:.5rem 1rem;border:0;margin:.2rem 0}.dismiss-btn:hover{background-color:#d60003}.oav-parameter{border-bottom:1px solid #ccc;padding:.2rem 0;box-sizing:border-box}.oav-parameter .input,.oav-parameter .name{margin:.2rem 0;padding-right:.5rem}.oav-parameter .input input:not([type=checkbox]),.oav-parameter .input select,.oav-parameter .input textarea{width:100%;padding:.2rem;font-size:1rem;font-family:monospace;box-sizing:border-box}.oav-parameter .input textarea{min-height:10em}.oav-parameter .description{margin:.2rem 0}.oav-parameter .description p{margin-top:0}.oav-parameter .description p:last-child{margin-bottom:0}.oav-parameter .required{color:#ac0003}.oav-parameter .format,.oav-parameter .param-in{color:#828282;font-size:.8rem}.multi-item-entry{display:-webkit-box;display:flex;margin-bottom:.2rem}.multi-item-entry input{padding:.2rem;font-size:1rem;font-family:monospace;box-sizing:border-box}.multi-item-entry button{color:#fff;background-color:#a7a7ac;font-size:1rem;padding:.2rem 1rem;border:0;min-width:2.6rem;margin-left:.2rem}.multi-item-entry button:hover{background-color:#d4d4d9}.oav-add-btn{color:#fff;background-color:#a7a7ac;font-size:1rem;padding:.2rem 1rem;border:0;min-width:2.6rem}.oav-add-btn:hover{background-color:#d4d4d9}.oav-btn-container{text-align:right}.oav-divided-container .main{-webkit-box-flex:1;flex:1}.oav-auth-status{font-family:monospace;cursor:pointer}.oav-auth-status::before{content:\" \"}.oav-auth-status.required::before{content:\"\uD83D\uDD12\uFE0E\";color:#8a8a8f}.oav-auth-status.ok::before{content:\"\uD83D\uDD13\uFE0E\";color:#25ac20}.oav-request{background:#f1f1f1;margin:.5rem 0}.oav-request.method-get{background-color:rgba(38,129,172,.1)}.oav-request.method-get h3:hover{background-color:rgba(38,129,172,.2)}.oav-request.method-post{background-color:rgba(37,172,32,.1)}.oav-request.method-post h3:hover{background-color:rgba(37,172,32,.2)}.oav-request.method-put{background-color:rgba(172,112,14,.1)}.oav-request.method-put h3:hover{background-color:rgba(172,112,14,.2)}.oav-request.method-delete{background-color:rgba(172,0,3,.1)}.oav-request.method-delete h3:hover{background-color:rgba(172,0,3,.2)}.oav-request.method-options{background-color:rgba(172,3,166,.1)}.oav-request.method-options h3:hover{background-color:rgba(172,3,166,.2)}.oav-request.method-head{background-color:rgba(135,170,172,.1)}.oav-request.method-head h3:hover{background-color:rgba(135,170,172,.2)}.oav-request.method-trace{background-color:rgba(135,172,137,.1)}.oav-request.method-trace h3:hover{background-color:rgba(135,172,137,.2)}.oav-request h4{background-color:rgba(255,255,255,.6);padding:.5rem 2rem;margin:.5rem -2rem}.oav-request .content{padding:1rem}.oav-request textarea{font-family:monospace;white-space:pre-wrap;box-sizing:border-box;background:#fff;width:100%;min-height:5rem}.oav-request .status-200,.oav-request .status-201,.oav-request .status-202,.oav-request .status-204{color:#25ac20}.oav-request .status-400,.oav-request .status-401,.oav-request .status-402,.oav-request .status-403,.oav-request .status-404,.oav-request .status-405,.oav-request .status-406,.oav-request .status-500,.oav-request .status-501,.oav-request .status-503,.oav-request .status-504,.oav-request .status-505{color:#ac0003}.oav-tab-nav{border-bottom:1px solid #ccc}.oav-tab-nav a{color:#000;display:inline-block;text-decoration:none;padding:.5rem 1rem}.oav-tab-nav a.active,.oav-tab-nav a:hover{color:#2681ac}.oav-tab-nav a.active{padding-bottom:.3rem;border-bottom:.2rem solid #2681ac}.oav-index-nav{padding:0 0 1rem;font-size:.8rem}.oav-index-nav .deprecated{text-decoration:line-through}.oav-index-nav header a{color:#000;text-decoration:none;padding:.2em 1rem;font-size:1rem;display:-webkit-box;display:flex;-webkit-box-pack:justify;justify-content:space-between;font-weight:700}.oav-index-nav header a:hover{color:#2681ac;background:#ebebeb}.oav-index-nav ol{padding-left:1rem}.oav-index-nav li{padding:0;display:block}.oav-index-nav li oav-method{margin-right:.5em}.oav-index-nav li a{color:#000;text-decoration:none;padding:.2em;box-sizing:border-box;display:block;position:relative}.oav-index-nav li a .label{display:-webkit-box;display:flex}.oav-index-nav li a .info-overlay{padding:.2em;display:none;position:absolute;background:#ebebeb;top:0;left:0;white-space:nowrap}.oav-index-nav li a:hover{background:#ebebeb;color:#2681ac}.oav-index-nav li a:hover .info-overlay{display:-webkit-box;display:flex}.oav-index-nav li a:hover .label{visibility:hidden}.oav-index-nav li a.active,.oav-index-nav li a:active{color:#2681ac}.oav-index-nav .oav-bottom-nav,.oav-index-nav .oav-top-nav{border-bottom:1px solid #ccc;margin:.5rem 0;padding:.5rem 0}.oav-index-nav .oav-bottom-nav a,.oav-index-nav .oav-top-nav a{font-size:1rem;padding-left:1rem}.oav-index-nav .oav-bottom-nav{border-bottom:0;border-top:1px solid #ccc}.m-0{margin:0}.my-0{margin-top:0;margin-bottom:0}.my-3{margin-top:1rem;margin-bottom:1rem}.mt-0{margin-top:0}.ml-auto{margin-left:auto}.ml-1{margin-left:.25rem}.ml-2{margin-left:.5rem}.ml-3{margin-left:1rem}.font-size-tiny{font-size:.5rem}.font-size-smaller{font-size:.8rem}.font-size-1{font-size:1rem}.font-size-2{font-size:2rem}.font-size-3{font-size:3rem}.d-flex{display:-webkit-box;display:flex}.align-items-center{-webkit-box-align:center;align-items:center}.flex-grow-1{-webkit-box-flex:1;flex:1}.justify-content-between{-webkit-box-pack:justify;justify-content:space-between}.normal-text{font-size:1rem;font-weight:400}.oav-error{color:#ac0003}.openable{cursor:pointer}.loading:before{content:\"\u21BB\";display:inline-block;-webkit-animation:2s linear infinite spin-loader;animation:2s linear infinite spin-loader}oav-tooltip{display:block;background:#fff;border:1px solid #ccc;padding:1rem;box-shadow:0 0 3px #ccc}@-webkit-keyframes spin-loader{to{-webkit-transform:rotate(360deg);transform:rotate(360deg)}}@keyframes spin-loader{to{-webkit-transform:rotate(360deg);transform:rotate(360deg)}}.openapi-response{border-bottom:1px solid #ccc;padding:.2rem 0;box-sizing:border-box}.openapi-response .status-code{margin:.2rem 0;padding-right:.5rem}.openapi-response .description{margin:.2rem 0}@media (min-width:60rem){.d-md-flex{display:-webkit-box;display:flex}.oav-layout .oav-menu-header{display:none}.oav-layout .oav-content-container .oav-index-container{margin-left:0}.oav-layout .oav-content-container .oav-view-container.open{margin-right:0}.oav-parameter{display:-webkit-box;display:flex}.oav-parameter .name{width:15rem}.oav-parameter .description,.oav-parameter .input{width:50%}.openapi-response{display:-webkit-box;display:flex}.openapi-response .status-code{width:15rem}.openapi-response .description{min-width:30rem}.oav-divided-container{display:-webkit-box;display:flex}}"]
            }] }
];
/** @nocollapse */
OpenapiViewerComponent.ctorParameters = () => [
    { type: OpenapiViewerService }
];
OpenapiViewerComponent.propDecorators = {
    spec: [{ type: Input }],
    specUrl: [{ type: Input }]
};
if (false) {
    /**
     * An already parsed spec. Is allowed to contain $ref.
     * @type {?}
     */
    OpenapiViewerComponent.prototype.spec;
    /**
     * Alternatively an URL to a spec to resolve.
     * @type {?}
     */
    OpenapiViewerComponent.prototype.specUrl;
    /** @type {?} */
    OpenapiViewerComponent.prototype.index;
    /** @type {?} */
    OpenapiViewerComponent.prototype.loading;
    /** @type {?} */
    OpenapiViewerComponent.prototype.error;
    /** @type {?} */
    OpenapiViewerComponent.prototype.loadErrors;
    /**
     * @type {?}
     * @private
     */
    OpenapiViewerComponent.prototype.destroy;
    /**
     * @type {?}
     * @private
     */
    OpenapiViewerComponent.prototype.openApiService;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class OavSettings {
    /**
     * @param {?=} overwriteSettings
     */
    constructor(overwriteSettings) {
        /**
         * Which value of an operation is shown in the index navigation, in which order.
         * The first one is displayed if available, otherwise the nex one will be  chosen until a value is available.
         */
        this.indexPrimaryLabel = ['summary', 'operationId', 'path'];
        /**
         * Which value is shown in the index navigation on hover.
         */
        this.indexHoverLabel = ['path'];
        /**
         * Which value of an operation is shown in the overview navigation, in which order.
         * The first one is displayed if available, otherwise the nex one will be  chosen until a value is available.
         */
        this.overviewPrimaryLabel = ['summary', 'operationId'];
        /**
         * Whether to show the option to display the raw operation definition on the operation page.
         */
        this.showRawOperationDefinition = false;
        /**
         * Show raw model definitions
         */
        this.showRawModelDefinition = false;
        /**
         * Whether authentication is enabled.
         * - Status is shown at operations
         * - Authentication page is shown, where a user can enter credentials
         */
        this.enableAuthentication = true;
        /**
         * Show the models overview page
         */
        this.enableModelsOverview = true;
        if (overwriteSettings) {
            Object.assign(this, overwriteSettings);
        }
    }
}
if (false) {
    /**
     * Which value of an operation is shown in the index navigation, in which order.
     * The first one is displayed if available, otherwise the nex one will be  chosen until a value is available.
     * @type {?}
     */
    OavSettings.prototype.indexPrimaryLabel;
    /**
     * Which value is shown in the index navigation on hover.
     * @type {?}
     */
    OavSettings.prototype.indexHoverLabel;
    /**
     * Which value of an operation is shown in the overview navigation, in which order.
     * The first one is displayed if available, otherwise the nex one will be  chosen until a value is available.
     * @type {?}
     */
    OavSettings.prototype.overviewPrimaryLabel;
    /**
     * Whether to show the option to display the raw operation definition on the operation page.
     * @type {?}
     */
    OavSettings.prototype.showRawOperationDefinition;
    /**
     * Show raw model definitions
     * @type {?}
     */
    OavSettings.prototype.showRawModelDefinition;
    /**
     * Whether authentication is enabled.
     * - Status is shown at operations
     * - Authentication page is shown, where a user can enter credentials
     * @type {?}
     */
    OavSettings.prototype.enableAuthentication;
    /**
     * Show the models overview page
     * @type {?}
     */
    OavSettings.prototype.enableModelsOverview;
}
/** @type {?} */
const defaultOavSettings = new OavSettings();

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class OperationComponent {
    /**
     * @param {?} openApiService
     * @param {?} oavSettings
     * @param {?} authService
     * @param {?} cd
     */
    constructor(openApiService, oavSettings, authService, cd) {
        this.openApiService = openApiService;
        this.oavSettings = oavSettings;
        this.authService = authService;
        this.cd = cd;
        this.responseType = new FormControl('application/json');
        this.requests = [];
        this.openRequest = null;
        this.showRawOperationDefinition = false;
        this.showAuthentication = true;
        if (!this.oavSettings) {
            this.oavSettings = defaultOavSettings;
        }
        this.showRawOperationDefinition = this.oavSettings.showRawOperationDefinition;
        this.showAuthentication = this.oavSettings.enableAuthentication;
    }
    /**
     * @return {?}
     */
    get operation() {
        return this.operationItem.operation;
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (changes.operationItem && this.operationItem) {
            if (changes.operationItem.previousValue && this.formGroup) {
                /** @type {?} */
                const prevOperation = changes.operationItem.previousValue;
                this.openApiService.saveOperationParameters(prevOperation.operation.operationId, this.formGroup.value);
            }
            this.requests = this.openApiService.getRequestsByOperationId(this.operationItem.operation.operationId).reverse();
            this.formGroup = new FormGroup({ __responseType: this.responseType });
            this.responseType.patchValue(this.operationItem.responseTypes[0] || 'application/json');
            this.authStatus = this.authService.getAuthStatus(this.operationItem.operation.security);
            /** @type {?} */
            const savedParams = this.openApiService.loadOperationParameters(this.operation.operationId);
            if (savedParams) {
                this.cd.detectChanges();
                this.formGroup.patchValue(savedParams);
            }
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.operationItem) {
            this.openApiService.saveOperationParameters(this.operation.operationId, this.formGroup.value);
        }
    }
    /**
     * @return {?}
     */
    send() {
        try {
            /** @type {?} */
            const parameters = Object.assign({}, this.formGroup.value);
            /** @type {?} */
            let requestBody;
            /** @type {?} */
            let contentType = '';
            if (parameters.requestBody) {
                contentType = parameters.requestBody.contentType;
                requestBody = Object.assign({}, parameters.requestBody);
                delete requestBody.contentType;
                delete parameters.requestBody;
            }
            /** @type {?} */
            const req = this.openApiService.createRequest(this.operationItem.operation.operationId, parameters, requestBody, contentType, this.responseType.value);
            /** @type {?} */
            const reqInfo = this.openApiService.runRequest(this.pathItem, this.operationItem, req);
            this.requests.unshift(reqInfo);
            this.openRequest = reqInfo;
        }
        catch (e) {
            console.warn('Create request error', e);
        }
    }
}
OperationComponent.decorators = [
    { type: Component, args: [{
                selector: 'oav-operation',
                template: "<h1>\n  <oav-method [method]=\"operationItem.method\"></oav-method>\n  <oav-auth-status [operation]=\"operationItem.operation\"></oav-auth-status>\n  <span class=\"oav-path ml-1\" [class.deprecated]=\"operation.deprecated\">{{ pathItem.path }}</span>\n  <span class=\"oav-summary font-size-smaller ml-2\">{{ operation.summary }}</span>\n</h1>\n\n<p *ngIf=\"operationItem.operation.description\">\n  <markdown [data]=\"operationItem.operation.description\"></markdown>\n</p>\n\n<form (submit)=\"send()\" [formGroup]=\"formGroup\">\n  <ng-container *ngIf=\"operationItem.parameters?.length\">\n    <h3>Parameters</h3>\n    <oav-parameter *ngFor=\"let param of operationItem.parameters\" [parameter]=\"param\" [formGroup]=\"formGroup\"></oav-parameter>\n  </ng-container>\n  <ng-container *ngIf=\"operationItem.operation.requestBody\">\n    <oav-request-body-input [requestBody]=\"operationItem.operation.requestBody\" [addToFormGroup]=\"formGroup\"></oav-request-body-input>\n  </ng-container>\n  <div *ngIf=\"operationItem.responseTypes?.length > 1\">\n    <div class=\"oav-parameter\">\n      <div class=\"name\">\n        <div><strong>Response type</strong>:</div>\n        <div class=\"param-in\">\n          Accept header\n        </div>\n      </div>\n      <div class=\"input\">\n        <select [formControl]=\"responseType\">\n          <option *ngFor=\"let resType of operationItem.responseTypes\" [value]=\"resType\">{{ resType }}</option>\n        </select>\n      </div>\n      <div class=\"description\"></div>\n    </div>\n  </div>\n  <button class=\"send-btn\" [disabled]=\"formGroup.invalid\">Send request</button>\n</form>\n\n<h3 oavOpenable #responseOpenable=\"openable\" class=\"toggle-button\">Responses</h3>\n\n<oav-responses\n  *ngIf=\"responseOpenable.open\"\n  [responses]=\"operationItem.operation.responses\"\n  [responseType]=\"responseType.value\"\n></oav-responses>\n\n<div class=\"oav-operation-requests\" [hidden]=\"!requests.length\">\n  <h3>Requests</h3>\n  <oav-requests-list [requests]=\"requests\" [openRequest]=\"openRequest\"></oav-requests-list>\n</div>\n\n<ng-container *ngIf=\"showRawOperationDefinition\">\n  <h3 oavOpenable #rawOpenable=\"openable\" class=\"toggle-button\">Raw operation definition</h3>\n  <oav-code-input *ngIf=\"rawOpenable.open\" mode=\"json\" [ngModel]=\"operationItem.operation | json\" readonly></oav-code-input>\n</ng-container>\n"
            }] }
];
/** @nocollapse */
OperationComponent.ctorParameters = () => [
    { type: OpenapiViewerService },
    { type: OavSettings, decorators: [{ type: Optional }] },
    { type: OpenapiAuthService },
    { type: ChangeDetectorRef }
];
OperationComponent.propDecorators = {
    tag: [{ type: Input }],
    pathItem: [{ type: Input }],
    operationItem: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    OperationComponent.prototype.tag;
    /** @type {?} */
    OperationComponent.prototype.pathItem;
    /** @type {?} */
    OperationComponent.prototype.operationItem;
    /** @type {?} */
    OperationComponent.prototype.responseType;
    /** @type {?} */
    OperationComponent.prototype.formGroup;
    /** @type {?} */
    OperationComponent.prototype.requests;
    /** @type {?} */
    OperationComponent.prototype.openRequest;
    /** @type {?} */
    OperationComponent.prototype.showRawOperationDefinition;
    /** @type {?} */
    OperationComponent.prototype.showAuthentication;
    /** @type {?} */
    OperationComponent.prototype.authStatus;
    /**
     * @type {?}
     * @private
     */
    OperationComponent.prototype.openApiService;
    /**
     * @type {?}
     * @private
     */
    OperationComponent.prototype.oavSettings;
    /**
     * @type {?}
     * @private
     */
    OperationComponent.prototype.authService;
    /**
     * @type {?}
     * @private
     */
    OperationComponent.prototype.cd;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @record
 */
function DisplayMode() { }
if (false) {
    /** @type {?} */
    DisplayMode.prototype.mode;
    /** @type {?} */
    DisplayMode.prototype.checkV2;
    /** @type {?} */
    DisplayMode.prototype.checkV3;
    /** @type {?} */
    DisplayMode.prototype.getDefault;
}
/** @type {?} */
const primitiveTypes = ['string', 'integer', 'number', 'boolean'];
const ɵ0$1 = /**
 * @param {?} param
 * @return {?}
 */
(param) => param.type === 'string' && param.items && param.items.enum, ɵ1$1 = /**
 * @param {?} param
 * @return {?}
 */
(param) => schemaType(param) === 'string' && ((/** @type {?} */ (param.schema))).enum, ɵ2$1 = /**
 * @param {?} param
 * @param {?} mediaType
 * @return {?}
 */
(param, mediaType) => getExample(param, mediaType), ɵ3$1 = /**
 * @param {?} param
 * @return {?}
 */
(param) => param.type === 'string', ɵ4$1 = /**
 * @param {?} param
 * @return {?}
 */
(param) => schemaType(param) === 'string', ɵ5$1 = /**
 * @param {?} param
 * @param {?} mediaType
 * @return {?}
 */
(param, mediaType) => getExample(param, mediaType), ɵ6$1 = /**
 * @param {?} param
 * @return {?}
 */
(param) => param.type === 'integer', ɵ7$1 = /**
 * @param {?} param
 * @return {?}
 */
(param) => schemaType(param) === 'integer', ɵ8$1 = /**
 * @param {?} param
 * @param {?} mediaType
 * @return {?}
 */
(param, mediaType) => getExample(param, mediaType), ɵ9$1 = /**
 * @param {?} param
 * @return {?}
 */
(param) => param.type === 'object', ɵ10$1 = /**
 * @param {?} param
 * @return {?}
 */
(param) => schemaType(param) === 'object' || (param.schema && ((/** @type {?} */ (param.schema))).properties), ɵ11$1 = /**
 * @param {?} param
 * @param {?} mediaType
 * @return {?}
 */
(param, mediaType) => getExample(param, mediaType), ɵ12$1 = /**
 * @param {?} param
 * @return {?}
 */
(param) => param.type === 'array' && param.items && param.items.enum, ɵ13$1 = /**
 * @param {?} param
 * @return {?}
 */
(param) => schemaType(param) === 'array' && ((/** @type {?} */ (param.schema))).enum, ɵ14$1 = /**
 * @param {?} param
 * @param {?} mediaType
 * @return {?}
 */
(param, mediaType) => getExample(param, mediaType), ɵ15 = /**
 * @param {?} param
 * @return {?}
 */
(param) => param.type === 'array' && primitiveTypes.includes(itemsType(param)), ɵ16 = /**
 * @param {?} param
 * @return {?}
 */
(param) => schemaType(param) === 'array' && primitiveTypes.includes(itemsType(param.schema)), ɵ17 = /**
 * @param {?} param
 * @param {?} mediaType
 * @return {?}
 */
(param, mediaType) => getExample(param, mediaType), ɵ18 = /**
 * @param {?} param
 * @return {?}
 */
(param) => param.type === 'array', ɵ19 = /**
 * @param {?} param
 * @return {?}
 */
(param) => schemaType(param) === 'array', ɵ20 = /**
 * @param {?} param
 * @param {?} mediaType
 * @return {?}
 */
(param, mediaType) => getExample(param, mediaType), ɵ21 = /**
 * @param {?} param
 * @return {?}
 */
(param) => param.type === 'file', ɵ22 = /**
 * @param {?} param
 * @return {?}
 */
(param) => false, ɵ23 = /**
 * @param {?} param
 * @param {?} mediaType
 * @return {?}
 */
(param, mediaType) => '', ɵ24 = /**
 * @param {?} param
 * @return {?}
 */
(param) => true, ɵ25 = /**
 * @param {?} param
 * @return {?}
 */
(param) => true, ɵ26 = /**
 * @param {?} param
 * @param {?} mediaType
 * @return {?}
 */
(param, mediaType) => '';
/** @type {?} */
const displayModes = [
    {
        mode: 'textSelect',
        checkV2: (ɵ0$1),
        checkV3: (ɵ1$1),
        getDefault: (ɵ2$1)
    },
    {
        mode: 'text',
        checkV2: (ɵ3$1),
        checkV3: (ɵ4$1),
        getDefault: (ɵ5$1)
    },
    {
        mode: 'integer',
        checkV2: (ɵ6$1),
        checkV3: (ɵ7$1),
        getDefault: (ɵ8$1)
    },
    {
        mode: 'object',
        checkV2: (ɵ9$1),
        checkV3: (ɵ10$1),
        getDefault: (ɵ11$1)
    },
    {
        mode: 'arrayWithSelection',
        checkV2: (ɵ12$1),
        checkV3: (ɵ13$1),
        getDefault: (ɵ14$1)
    },
    {
        mode: 'arrayWithPrimitive',
        checkV2: (ɵ15),
        checkV3: (ɵ16),
        getDefault: (ɵ17)
    },
    {
        mode: 'array',
        checkV2: (ɵ18),
        checkV3: (ɵ19),
        getDefault: (ɵ20)
    },
    {
        mode: 'file',
        checkV2: (ɵ21),
        checkV3: (ɵ22),
        getDefault: (ɵ23)
    },
    {
        mode: 'any',
        checkV2: (ɵ24),
        checkV3: (ɵ25),
        getDefault: (ɵ26)
    }
];
/**
 * @param {?} param
 * @return {?}
 */
function schemaType(param) {
    /** @type {?} */
    const schema = param && 'schema' in param ? ((/** @type {?} */ (param.schema))) : null;
    return schema && schema.type;
}
/**
 * @param {?} obj
 * @return {?}
 */
function itemsType(obj) {
    if (obj && Array.isArray(obj.items)) {
        return obj.items[0].type;
    }
    else if (obj && obj.items) {
        return obj.items.type;
    }
    console.warn('items type not found', obj);
}
/**
 * @param {?} param
 * @param {?} mediaType
 * @return {?}
 */
function getExample(param, mediaType) {
    if (param.example !== undefined) {
        return param.example;
    }
    if (param.examples !== undefined && param.examples[mediaType] && 'value' in param.examples[mediaType]) {
        return ((/** @type {?} */ (param.examples[mediaType]))).value;
    }
    if (param.schema) {
        return exampleFromSchema(param.schema);
    }
    if (param.type === 'array' && param.items && 'default' in param.items) {
        return [param.items.default];
    }
    if (isPrimitiveType(param.type) && param.items && 'default' in param.items) {
        return param.items.default;
    }
    return exampleFromSchema((/** @type {?} */ (param)));
}
/**
 * @param {?} param
 * @return {?}
 */
function getDisplayMode(param) {
    for (const dm of displayModes) {
        if ('type' in param && dm.checkV2(param)) {
            return dm.mode;
        }
        else if (dm.checkV3(param)) {
            return dm.mode;
        }
    }
    return 'any';
}
/**
 * @param {?} displayMode
 * @param {?} param
 * @param {?} mediaType
 * @return {?}
 */
function getExampleValue(displayMode, param, mediaType) {
    return displayModes.find((/**
     * @param {?} dm
     * @return {?}
     */
    dm => dm.mode === displayMode)).getDefault(param, mediaType);
}
/**
 * @param {?} type
 * @return {?}
 */
function isPrimitiveType(type) {
    return ['number', 'boolean', 'string', 'integer'].includes(type);
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ParameterComponent {
    constructor() {
        this.mediaType = '';
        this.displayMode = 'any';
        this.enum = null;
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        this.displayMode = getDisplayMode(this.parameter);
        this.value = getExampleValue(this.displayMode, this.parameter, this.mediaType);
        if (this.displayMode === 'object' || this.displayMode === 'array') {
            this.value = JSON.stringify(this.value, null, 2);
        }
        /** @type {?} */
        const validators = [];
        if (this.parameter.required) {
            validators.push(Validators.required);
        }
        this.control = new FormControl(this.value, validators);
        if (this.formGroup) {
            this.formGroup.addControl(this.parameter.name, this.control);
        }
        this.enum =
            (this.parameter.items && this.parameter.items.enum) || (this.parameter.schema && ((/** @type {?} */ (this.parameter.schema))).enum);
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.formGroup) {
            this.formGroup.removeControl(this.parameter.name);
        }
    }
}
ParameterComponent.decorators = [
    { type: Component, args: [{
                selector: 'oav-parameter',
                template: "<div class=\"oav-parameter\" [formGroup]=\"formGroup\">\n  <div class=\"name\">\n    <div>\n      <strong>{{ parameter.name }}</strong\n      ><span class=\"required\" *ngIf=\"parameter.required\" title=\"required\">*</span>:\n      <span class=\"type\">{{ parameter.type }}</span>\n    </div>\n    <div class=\"param-in\">\n      {{ parameter.customIn ? parameter.customIn : parameter.in }}\n      <span class=\"format\" *ngIf=\"parameter.format\">({{ parameter.format }})</span>\n    </div>\n  </div>\n  <div class=\"input\">\n    <ng-container *ngIf=\"displayMode === 'textSelect'\">\n      <select [formControlName]=\"parameter.name\" [name]=\"parameter.name\">\n        <option *ngFor=\"let opt of enum\" [value]=\"opt\">{{ opt }}</option>\n      </select>\n    </ng-container>\n\n    <ng-container *ngIf=\"displayMode === 'text'\">\n      <input [formControlName]=\"parameter.name\" [placeholder]=\"parameter.name\" [name]=\"parameter.name\" />\n    </ng-container>\n\n    <ng-container *ngIf=\"displayMode === 'integer'\">\n      <input type=\"number\" [formControlName]=\"parameter.name\" [placeholder]=\"parameter.name\" [name]=\"parameter.name\" />\n    </ng-container>\n\n    <ng-container *ngIf=\"displayMode === 'number'\">\n      <input type=\"number\" [formControlName]=\"parameter.name\" [placeholder]=\"parameter.name\" [name]=\"parameter.name\" />\n    </ng-container>\n\n    <ng-container *ngIf=\"displayMode === 'file'\">\n      <input type=\"file\" [formControlName]=\"parameter.name\" [placeholder]=\"parameter.name\" [name]=\"parameter.name\" />\n    </ng-container>\n\n    <ng-container *ngIf=\"displayMode === 'object' || displayMode === 'array'\">\n      <oav-code-input [formControlName]=\"parameter.name\" mode=\"json\" [schema]=\"parameter.schema\"></oav-code-input>\n    </ng-container>\n\n    <ng-container *ngIf=\"displayMode === 'arrayWithSelection'\">\n      <select [formControlName]=\"parameter.name\" [name]=\"parameter.name\" multiple>\n        <option *ngFor=\"let opt of enum\" [ngValue]=\"opt\">{{ opt }}</option>\n      </select>\n    </ng-container>\n\n    <ng-container *ngIf=\"displayMode === 'arrayWithPrimitive'\">\n      <oav-multi-items-input [formControlName]=\"parameter.name\"></oav-multi-items-input>\n    </ng-container>\n  </div>\n  <div class=\"description\">\n    <markdown [data]=\"parameter.description\"></markdown>\n\n    <ng-container *ngIf=\"displayMode === 'object' || displayMode === 'array'\">\n      <oav-json-schema [schema]=\"parameter.schema\"></oav-json-schema>\n    </ng-container>\n  </div>\n</div>\n"
            }] }
];
/** @nocollapse */
ParameterComponent.ctorParameters = () => [];
ParameterComponent.propDecorators = {
    formGroup: [{ type: Input }],
    parameter: [{ type: Input }],
    mediaType: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    ParameterComponent.prototype.formGroup;
    /** @type {?} */
    ParameterComponent.prototype.parameter;
    /** @type {?} */
    ParameterComponent.prototype.mediaType;
    /** @type {?} */
    ParameterComponent.prototype.displayMode;
    /** @type {?} */
    ParameterComponent.prototype.value;
    /** @type {?} */
    ParameterComponent.prototype.control;
    /** @type {?} */
    ParameterComponent.prototype.enum;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @record
 */
function ResponseContentItem() { }
if (false) {
    /** @type {?|undefined} */
    ResponseContentItem.prototype.name;
    /** @type {?|undefined} */
    ResponseContentItem.prototype.summary;
    /** @type {?|undefined} */
    ResponseContentItem.prototype.description;
    /** @type {?|undefined} */
    ResponseContentItem.prototype.value;
    /** @type {?|undefined} */
    ResponseContentItem.prototype.externalValueUrl;
    /** @type {?|undefined} */
    ResponseContentItem.prototype.schema;
}
/**
 * @record
 */
function ResponseHeaderExample() { }
if (false) {
    /** @type {?} */
    ResponseHeaderExample.prototype.name;
    /** @type {?|undefined} */
    ResponseHeaderExample.prototype.description;
    /** @type {?|undefined} */
    ResponseHeaderExample.prototype.deprecated;
    /** @type {?} */
    ResponseHeaderExample.prototype.value;
}
/**
 * @record
 */
function ResponseCodeItem() { }
if (false) {
    /** @type {?} */
    ResponseCodeItem.prototype.statusCode;
    /** @type {?} */
    ResponseCodeItem.prototype.description;
    /** @type {?|undefined} */
    ResponseCodeItem.prototype.headers;
    /** @type {?|undefined} */
    ResponseCodeItem.prototype.contents;
}
class ResponsesComponent {
    constructor() {
        this.items = [];
        this.mode = 'text';
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (!this.responses) {
            return;
        }
        if (this.responseType && this.responseType.startsWith('application/json')) {
            this.mode = 'json';
        }
        else if (this.responseType && this.responseType.startsWith('application/xml')) {
            this.mode = 'xml';
        }
        else {
            this.mode = 'text';
        }
        this.items = getExamplesByContentType(this.responses, this.responseType);
    }
}
ResponsesComponent.decorators = [
    { type: Component, args: [{
                selector: 'oav-responses',
                template: "<oav-tab-nav>\n  <ng-container *ngFor=\"let status of items\">\n    <div *oavTabNavItem=\"status.statusCode\">\n      <markdown *ngIf=\"status.description\" [data]=\"status.description\"></markdown>\n      <ng-container *ngIf=\"status.headers?.length\">\n        <h4>Example Headers</h4>\n        <div class=\"my-3\">\n          <div class=\"oav-parameter\" *ngFor=\"let header of status.headers\">\n            <div class=\"name\">{{ header.name }}:</div>\n            <div class=\"input\">{{ header.value }}</div>\n            <div class=\"description\">\n              <markdown [data]=\"header.description\"></markdown>\n            </div>\n          </div>\n        </div>\n      </ng-container>\n      <ng-container *ngIf=\"status.contents?.length === 1\">\n        <h4>Example Body</h4>\n        <div class=\"oav-divided-container\">\n          <div class=\"flex-grow-1\">\n            <oav-code-input readonly [ngModel]=\"status.contents[0].value\" [mode]=\"mode\"></oav-code-input>\n          </div>\n          <div *ngIf=\"status.contents[0].schema\" class=\"flex-grow-1 ml-3\">\n            <oav-json-schema [schema]=\"status.contents[0].schema\"></oav-json-schema>\n          </div>\n        </div>\n      </ng-container>\n      <ng-container *ngIf=\"status.contents?.length > 1\">\n        <oav-tab-nav>\n          <ng-container *ngFor=\"let content of status.contents\">\n            <div *oavTabNavItem=\"content.summary || content.name\">\n              <markdown *ngIf=\"content.description\" [data]=\"content.description\"></markdown>\n              <a *ngIf=\"content.externalValueUrl\" [href]=\"content.externalValueUrl\" target=\"_blank\">Example</a>\n              <div class=\"oav-divided-container\" *ngIf=\"!content.externalValueUrl\">\n                <div class=\"flex-grow-1\">\n                  <oav-code-input readonly [ngModel]=\"content.value\" [mode]=\"mode\" [maxLines]=\"20\"></oav-code-input>\n                </div>\n                <div *ngIf=\"content.schema\" class=\"flex-grow-1 ml-3\">\n                  <oav-json-schema [schema]=\"content.schema\"></oav-json-schema>\n                </div>\n              </div>\n            </div>\n          </ng-container>\n        </oav-tab-nav>\n      </ng-container>\n    </div>\n  </ng-container>\n</oav-tab-nav>\n"
            }] }
];
/** @nocollapse */
ResponsesComponent.ctorParameters = () => [];
ResponsesComponent.propDecorators = {
    responses: [{ type: Input }],
    responseType: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    ResponsesComponent.prototype.responses;
    /** @type {?} */
    ResponsesComponent.prototype.responseType;
    /** @type {?} */
    ResponsesComponent.prototype.items;
    /** @type {?} */
    ResponsesComponent.prototype.mode;
}
/**
 * @param {?} responses
 * @param {?} contentType
 * @return {?}
 */
function getExamplesByContentType(responses, contentType) {
    return Object.keys(responses).map((/**
     * @param {?} statusCode
     * @return {?}
     */
    statusCode => {
        /** @type {?} */
        const res = responses[statusCode];
        return {
            statusCode,
            description: res.description,
            headers: getExampleHeaders(res.headers),
            contents: getExampleContents(res, contentType)
        };
    }));
}
/**
 * @param {?} res
 * @param {?} contentType
 * @return {?}
 */
function getExampleContents(res, contentType) {
    if (res.schema) {
        return [
            {
                value: JSON.stringify(exampleFromSchema(res.schema), null, 2),
                schema: res.schema
            }
        ];
    }
    if (!res.content) {
        return [];
    }
    /** @type {?} */
    let content = res.content[contentType];
    if (!content) {
        /** @type {?} */
        const mediaType = Object.keys(res.content).find((/**
         * @param {?} mt
         * @return {?}
         */
        mt => getStarMatcher(mt).test(contentType)));
        if (mediaType) {
            content = res.content[mediaType];
        }
    }
    if (!content) {
        return [];
    }
    if (content.example && contentType.includes('json')) {
        return [{ value: getJsonString(content.example) }];
    }
    if (content.examples) {
        return Object.keys(content.examples).map((/**
         * @param {?} exName
         * @return {?}
         */
        exName => {
            /** @type {?} */
            const example = content.examples[exName];
            return {
                name: exName,
                summary: example.summary,
                description: example.description,
                value: getJsonString(example.value),
                externalValueUrl: example.externalValue
            };
        }));
    }
    if (content.schema) {
        return [
            {
                value: JSON.stringify(exampleFromSchema(content.schema), null, 2),
                schema: content.schema
            }
        ];
    }
    return [];
}
/**
 * @param {?} headers
 * @return {?}
 */
function getExampleHeaders(headers) {
    if (!headers) {
        return [];
    }
    return Object.keys(headers).map((/**
     * @param {?} headerName
     * @return {?}
     */
    headerName => {
        /** @type {?} */
        const header = headers[headerName];
        return {
            name: headerName,
            description: header.description,
            deprecated: header.deprecated,
            value: getHeaderExampleValue(header)
        };
    }));
}
/**
 * @param {?} headerObject
 * @return {?}
 */
function getHeaderExampleValue(headerObject) {
    /** @type {?} */
    const displayMode = getDisplayMode(headerObject);
    return getExampleValue(displayMode, headerObject, null);
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class MultiItemsInputComponent {
    constructor() {
        this.disabled = false;
        this.values = new FormArray([]);
        this.onChange = (/**
         * @param {?} v
         * @return {?}
         */
        (v) => { });
        this.onTouched = (/**
         * @return {?}
         */
        () => { });
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.subscription = this.values.valueChanges.subscribe((/**
         * @param {?} changedValues
         * @return {?}
         */
        changedValues => {
            this.onChange(changedValues);
            this.onTouched();
        }));
        /** @type {?} */
        const controls = {};
        controls[this.name] = this.values;
        this.formGroup = new FormGroup(controls);
    }
    /**
     * @return {?}
     */
    get isRequired() {
        return typeof this.required !== 'undefined' || this.required === true;
    }
    /**
     * @return {?}
     */
    get controls() {
        return this.values.controls;
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.subscription) {
            this.subscription.unsubscribe();
        }
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    /**
     * @param {?} isDisabled
     * @return {?}
     */
    setDisabledState(isDisabled) {
        this.disabled = isDisabled;
        if (this.disabled) {
            this.values.disable();
        }
        else {
            this.values.enable();
        }
    }
    /**
     * @param {?} obj
     * @return {?}
     */
    writeValue(obj) {
        if (obj) {
            this.values.clear();
            obj.forEach((/**
             * @param {?} value
             * @return {?}
             */
            value => this.values.push(new FormControl({ value, disabled: this.disabled }))));
        }
        else {
            this.values.clear();
        }
    }
    /**
     * @return {?}
     */
    addControl() {
        this.values.push(new FormControl(''));
        this.onTouched();
    }
    /**
     * @param {?} index
     * @return {?}
     */
    removeValue(index) {
        this.values.removeAt(index);
        this.onTouched();
    }
}
MultiItemsInputComponent.decorators = [
    { type: Component, args: [{
                selector: 'oav-multi-items-input',
                template: "<ng-container [formGroup]=\"formGroup\">\n  <div [formArrayName]=\"name\">\n    <div class=\"multi-item-entry\" *ngFor=\"let control of controls; let i = index\">\n      <input type=\"text\" [formControlName]=\"i.toString()\" (blur)=\"onTouched()\" />\n      <button type=\"button\" (click)=\"removeValue(i)\" [disabled]=\"disabled\">-</button>\n    </div>\n  </div>\n  <div class=\"oav-btn-container\">\n    <button type=\"button\" (click)=\"addControl()\" class=\"oav-add-btn\" [disabled]=\"disabled\">+</button>\n  </div>\n</ng-container>\n",
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef((/**
                         * @return {?}
                         */
                        () => MultiItemsInputComponent)),
                        multi: true
                    }
                ]
            }] }
];
/** @nocollapse */
MultiItemsInputComponent.ctorParameters = () => [];
MultiItemsInputComponent.propDecorators = {
    name: [{ type: Input }],
    required: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    MultiItemsInputComponent.prototype.name;
    /** @type {?} */
    MultiItemsInputComponent.prototype.required;
    /** @type {?} */
    MultiItemsInputComponent.prototype.disabled;
    /** @type {?} */
    MultiItemsInputComponent.prototype.values;
    /** @type {?} */
    MultiItemsInputComponent.prototype.formGroup;
    /**
     * @type {?}
     * @private
     */
    MultiItemsInputComponent.prototype.subscription;
    /**
     * @type {?}
     * @private
     */
    MultiItemsInputComponent.prototype.onChange;
    /** @type {?} */
    MultiItemsInputComponent.prototype.onTouched;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class RequestsListComponent {
    /**
     * @param {?} openApiService
     */
    constructor(openApiService) {
        this.openApiService = openApiService;
        this.openRequests = new Set();
        this.requests = [];
    }
    /**
     * @return {?}
     */
    ngOnInit() { }
    /**
     * @param {?} req
     * @return {?}
     */
    set openRequest(req) {
        if (req) {
            this.openRequests.clear();
            this.openRequests.add(req);
        }
    }
    /**
     * @param {?} req
     * @param {?} state
     * @return {?}
     */
    toggleRequest(req, state) {
        if (this.openRequests.has(req)) {
            this.openRequests.delete(req);
        }
        else {
            this.openRequests.add(req);
        }
    }
    /**
     * @param {?} req
     * @return {?}
     */
    getShortUrl(req) {
        if (req.request.url.match(/^.+?\/\/.+?(\/[^?]*)/)) {
            return RegExp.$1;
        }
        return req.request.url;
    }
    /**
     * @param {?} req
     * @return {?}
     */
    removeRequest(req) {
        this.requests.splice(this.requests.indexOf(req), 1);
        this.openRequests.delete(req);
        this.openApiService.removeRequest(req);
    }
}
RequestsListComponent.decorators = [
    { type: Component, args: [{
                selector: 'oav-requests-list',
                template: "<div *ngFor=\"let req of requests\" [className]=\"'oav-request method-' + req.request.method.toLowerCase()\" [id]=\"req.id\">\n  <oav-method-header\n    [method]=\"req.request.method\"\n    [path]=\"getShortUrl(req)\"\n    [open]=\"openRequests.has(req)\"\n    (openChange)=\"toggleRequest(req, $event)\"\n  >\n    {{ req.startTs | date: 'medium' }}\n    <strong class=\"ml-2\">\n      <span [ngClass]=\"'status status-' + req.status\" *ngIf=\"!req.running && req.status\">{{ req.status }} {{ req.statusText }}</span>\n      <span class=\"loading\" *ngIf=\"req.running\"></span>\n    </strong>\n  </oav-method-header>\n\n  <oav-request-view [request]=\"req\" [hidden]=\"!openRequests.has(req)\" (dismiss)=\"removeRequest(req)\"></oav-request-view>\n</div>\n"
            }] }
];
/** @nocollapse */
RequestsListComponent.ctorParameters = () => [
    { type: OpenapiViewerService }
];
RequestsListComponent.propDecorators = {
    requests: [{ type: Input }],
    openRequest: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    RequestsListComponent.prototype.openRequests;
    /** @type {?} */
    RequestsListComponent.prototype.requests;
    /**
     * @type {?}
     * @private
     */
    RequestsListComponent.prototype.openApiService;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class RequestViewComponent {
    constructor() {
        this.dismiss = new EventEmitter();
        this.displayLimitBytes = 1024 * 1024;
        this.displayMode = 'text';
        this.destroy = new Subject();
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.loadResponse();
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.destroy.complete();
    }
    /**
     * @return {?}
     */
    loadResponse() {
        this.error = null;
        this.headers = '';
        this.body = '';
        this.request.httpEvents.pipe(takeUntil(this.destroy)).subscribe((/**
         * @param {?} status
         * @return {?}
         */
        status => {
            if (status.type === HttpEventType.ResponseHeader || status.type === HttpEventType.Response) {
                this.setHeaders(status.headers);
            }
            if (status.type === HttpEventType.Response) {
                this.setBody(status);
            }
        }), (/**
         * @param {?} err
         * @return {?}
         */
        err => {
            console.error('error response', err);
            if (err instanceof HttpErrorResponse) {
                this.setErrorBody(err);
            }
            this.error = err.message;
        }));
    }
    /**
     * @param {?} err
     * @return {?}
     */
    setErrorBody(err) {
        return __awaiter(this, void 0, void 0, function* () {
            if (err.error instanceof Blob) {
                this.updateDisplayMode(err.error.type);
                this.body = yield getTextFromBlob(err.error);
            }
        });
    }
    /**
     * @param {?} mimeType
     * @return {?}
     */
    updateDisplayMode(mimeType) {
        if (mimeType.startsWith('text')) {
            this.displayMode = 'text';
        }
        else if (mimeType.includes('json')) {
            this.displayMode = 'json';
        }
        else if (mimeType.includes('xml')) {
            this.displayMode = 'xml';
        }
        else {
            this.displayMode = 'download';
        }
    }
    /**
     * @param {?} response
     * @return {?}
     */
    setBody(response) {
        return __awaiter(this, void 0, void 0, function* () {
            this.blob = response.body;
            /** @type {?} */
            const contentType = response.headers.get('content-type');
            /** @type {?} */
            const textTypes = /^(application\/.{0,30}(json|xml)|text\/)/;
            if (contentType.match(textTypes) && this.displayLimitBytes > this.blob.size) {
                this.displayMode = 'text';
                /** @type {?} */
                const body = yield getTextFromBlob(response.body);
                if (contentType.startsWith('application/json') && body && body.length) {
                    this.displayMode = 'json';
                    try {
                        /** @type {?} */
                        const parsed = JSON.parse(body);
                        this.body = JSON.stringify(parsed, null, 2);
                    }
                    catch (e) {
                        this.body = body;
                        this.error = e;
                    }
                }
                else {
                    if (contentType.startsWith('application/xml')) {
                        this.displayMode = 'xml';
                    }
                    this.body = body;
                }
            }
            else {
                this.displayMode = 'download';
            }
        });
    }
    /**
     * @param {?} headers
     * @return {?}
     */
    setHeaders(headers) {
        this.httpHeaders = headers;
        /** @type {?} */
        const headerLines = [];
        for (const headerName of headers.keys()) {
            for (const value of headers.getAll(headerName)) {
                headerLines.push(headerName + ': ' + value);
            }
        }
        this.headers = headerLines.join('\n');
    }
    /**
     * @return {?}
     */
    getDuration() {
        if (this.request.endTs) {
            return this.request.endTs.getTime() - this.request.startTs.getTime();
        }
        else if (this.request.running) {
            return new Date().getTime() - this.request.startTs.getTime();
        }
        else {
            return null;
        }
    }
    /**
     * @return {?}
     */
    doDismiss() {
        if (this.request.running) {
            this.request.cancel();
        }
        this.dismiss.next();
    }
    /**
     * @return {?}
     */
    download() {
        /** @type {?} */
        const contentType = this.httpHeaders.get('content-type');
        /** @type {?} */
        const contentDisposition = this.httpHeaders.get('content-disposition');
        /** @type {?} */
        const contentType2Ending = {
            'application/json': 'json',
            'application/xml': 'xml',
            'text/plain': 'txt',
            'image/png': 'png',
            'image/gif': 'gif',
            'image/jpeg': 'jpg'
        };
        /** @type {?} */
        const readableTs = new Date().toISOString().replace(/[-:]|\.\d{3}/g, '');
        /** @type {?} */
        let name = readableTs + '_' + this.request.operationsItem.operation.operationId;
        /** @type {?} */
        const mime = Object.keys(contentType2Ending).find((/**
         * @param {?} mimeType
         * @return {?}
         */
        mimeType => contentType.startsWith(mimeType)));
        if (mime) {
            name += '.' + contentType2Ending[mime];
        }
        if (contentDisposition && contentDisposition.match(/filename="?(\w+)"?/)) {
            name = RegExp.$1;
        }
        /** @type {?} */
        const link = document.createElement('a');
        /** @type {?} */
        const url = URL.createObjectURL(this.blob);
        link.href = url;
        link.download = name;
        link.click();
        URL.revokeObjectURL(url);
    }
}
RequestViewComponent.decorators = [
    { type: Component, args: [{
                selector: 'oav-request-view',
                template: "<div class=\"oav-request-details\">\n  <div class=\"content\">\n    <div *ngIf=\"request.request.url\">\n      <div>\n        URL: <code>{{ request.request.url }}</code>\n      </div>\n    </div>\n    <div>\n      <div>\n        Status: <code>{{ request.status }} {{ request.statusText }}</code>\n      </div>\n    </div>\n    <div *ngIf=\"request.sentBytes\">\n      <div>\n        Bytes sent: <code>{{ request.sentBytes | number }}</code>\n      </div>\n    </div>\n\n    <h4>Response headers</h4>\n    <pre>{{ headers }}</pre>\n\n    <h4>\n      <span>Response body</span>\n      <span class=\"normal-text ml-2\">{{ request.receivedBytes | number }} bytes received after {{ getDuration() | number }} ms</span>\n    </h4>\n\n    <ng-container *ngIf=\"(displayMode === 'text' || displayMode === 'json' || displayMode === 'xml') && !request.running\">\n      <oav-code-input [ngModel]=\"body\" [mode]=\"displayMode\" [minLines]=\"5\" readonly></oav-code-input>\n    </ng-container>\n\n    <p class=\"oav-warning\" *ngIf=\"request.receivedBytes >= displayLimitBytes\">\n      Response is too large to be displayed. Please download it.\n    </p>\n\n    <pre *ngIf=\"error\" class=\"oav-error\">{{ error }}</pre>\n\n    <div class=\"d-flex justify-content-between\">\n      <button type=\"button\" class=\"download-btn\" (click)=\"download()\" *ngIf=\"!request.running\">Download</button>\n      <button type=\"button\" class=\"dismiss-btn\" (click)=\"doDismiss()\">{{ request.running ? 'Cancel' : 'Dismiss' }}</button>\n    </div>\n  </div>\n</div>\n"
            }] }
];
/** @nocollapse */
RequestViewComponent.ctorParameters = () => [];
RequestViewComponent.propDecorators = {
    request: [{ type: Input }],
    dismiss: [{ type: Output }]
};
if (false) {
    /** @type {?} */
    RequestViewComponent.prototype.request;
    /** @type {?} */
    RequestViewComponent.prototype.dismiss;
    /** @type {?} */
    RequestViewComponent.prototype.blob;
    /** @type {?} */
    RequestViewComponent.prototype.httpHeaders;
    /** @type {?} */
    RequestViewComponent.prototype.body;
    /** @type {?} */
    RequestViewComponent.prototype.headers;
    /** @type {?} */
    RequestViewComponent.prototype.displayLimitBytes;
    /** @type {?} */
    RequestViewComponent.prototype.displayMode;
    /** @type {?} */
    RequestViewComponent.prototype.error;
    /**
     * @type {?}
     * @private
     */
    RequestViewComponent.prototype.destroy;
}
/**
 * @param {?} blob
 * @return {?}
 */
function getTextFromBlob(blob) {
    return new Promise((/**
     * @param {?} resolve
     * @param {?} reject
     * @return {?}
     */
    (resolve, reject) => {
        /** @type {?} */
        const reader = new FileReader();
        reader.onload = (/**
         * @param {?} ev
         * @return {?}
         */
        (ev) => {
            resolve(ev.target.result);
        });
        reader.onerror = (/**
         * @param {?} err
         * @return {?}
         */
        err => {
            reject(err);
        });
        reader.readAsText(blob);
    }));
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class OpenableDirective {
    /**
     * @param {?} element
     * @param {?} renderer
     */
    constructor(element, renderer) {
        this.element = element;
        this.renderer = renderer;
        this.oavOpenable = true;
        this.open = false;
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        if (this.oavOpenable !== false) {
            this.renderer.addClass(this.element.nativeElement, 'openable');
        }
    }
    /**
     * @return {?}
     */
    onClick() {
        if (this.oavOpenable === false) {
            return;
        }
        this.open = !this.open;
        if (this.open) {
            this.renderer.addClass(this.element.nativeElement, 'open');
        }
        else {
            this.renderer.removeClass(this.element.nativeElement, 'open');
        }
    }
}
OpenableDirective.decorators = [
    { type: Directive, args: [{
                selector: '[oavOpenable]',
                exportAs: 'openable'
            },] }
];
/** @nocollapse */
OpenableDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 }
];
OpenableDirective.propDecorators = {
    oavOpenable: [{ type: Input }],
    open: [{ type: Input }],
    onClick: [{ type: HostListener, args: ['click',] }]
};
if (false) {
    /** @type {?} */
    OpenableDirective.prototype.oavOpenable;
    /** @type {?} */
    OpenableDirective.prototype.open;
    /**
     * @type {?}
     * @private
     */
    OpenableDirective.prototype.element;
    /**
     * @type {?}
     * @private
     */
    OpenableDirective.prototype.renderer;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class MethodHeaderComponent {
    constructor() {
        this.open = false;
        this.openChange = new EventEmitter();
    }
    /**
     * @return {?}
     */
    ngOnInit() { }
    /**
     * @return {?}
     */
    toggleOpen() {
        this.open = !this.open;
        this.openChange.next(this.open);
    }
}
MethodHeaderComponent.decorators = [
    { type: Component, args: [{
                selector: 'oav-method-header',
                template: "<h3 (click)=\"toggleOpen()\" [className]=\"'oav-method-header oav-method-bg method-' + method.toLowerCase()\">\n  <a href=\"javascript:\" class=\"mainlink\" (click)=\"toggleOpen(); $event.stopPropagation()\">\n    <oav-method [method]=\"method\"></oav-method>\n    <span class=\"path ml-2\">{{ path }}</span>\n  </a>\n  <span class=\"summary\">\n    <ng-content></ng-content>\n  </span>\n  <button type=\"button\" class=\"toggle-button\" [class.open]=\"open\"></button>\n</h3>\n"
            }] }
];
/** @nocollapse */
MethodHeaderComponent.ctorParameters = () => [];
MethodHeaderComponent.propDecorators = {
    open: [{ type: Input }],
    openChange: [{ type: Output }],
    fragment: [{ type: Input }],
    method: [{ type: Input }],
    path: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    MethodHeaderComponent.prototype.open;
    /** @type {?} */
    MethodHeaderComponent.prototype.openChange;
    /** @type {?} */
    MethodHeaderComponent.prototype.fragment;
    /** @type {?} */
    MethodHeaderComponent.prototype.method;
    /** @type {?} */
    MethodHeaderComponent.prototype.path;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class MethodComponent {
}
MethodComponent.decorators = [
    { type: Component, args: [{
                selector: 'oav-method',
                template: "<span [ngClass]=\"'oav-method oav-method-' + method.toLowerCase()\">{{ method.toUpperCase() }}</span>\n",
                changeDetection: ChangeDetectionStrategy.OnPush
            }] }
];
MethodComponent.propDecorators = {
    method: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    MethodComponent.prototype.method;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class IndexNavComponent {
    /**
     * @param {?} router
     * @param {?} oavSettings
     * @param {?} cd
     */
    constructor(router, oavSettings, cd) {
        this.router = router;
        this.oavSettings = oavSettings;
        this.cd = cd;
        this.openTags = new Set();
        if (!this.oavSettings) {
            this.oavSettings = defaultOavSettings;
        }
        this.showHoverLabel = !!this.oavSettings.indexHoverLabel;
        this.showAuthentication = this.oavSettings.enableAuthentication;
        this.showModelsOverview = this.oavSettings.enableModelsOverview;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.checkOpenTags(this.router.routerState.snapshot.url);
        this.sub = this.router.events.pipe(filter((/**
         * @param {?} ev
         * @return {?}
         */
        ev => ev instanceof NavigationEnd))).subscribe((/**
         * @param {?} event
         * @return {?}
         */
        (event) => {
            this.checkOpenTags(event.url);
            this.cd.markForCheck();
        }));
        if (this.index && this.index.length === 1) {
            this.openTags.add(this.index[0].tag.name);
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.sub.unsubscribe();
    }
    /**
     * @param {?} url
     * @return {?}
     */
    checkOpenTags(url) {
        if (url.match(/\/(.+)\/(.+)/)) {
            /** @type {?} */
            const tag = RegExp.$1;
            this.openTags.add(tag);
        }
    }
}
IndexNavComponent.decorators = [
    { type: Component, args: [{
                selector: 'oav-index-nav',
                template: "<nav class=\"oav-index-nav\">\n  <ol class=\"oav-top-nav\">\n    <li>\n      <a routerLink=\"\" routerLinkActive=\"active\" [routerLinkActiveOptions]=\"{ exact: true }\">Overview</a>\n    </li>\n    <li *ngIf=\"showAuthentication\">\n      <a routerLink=\"/auth\" routerLinkActive=\"active\" [routerLinkActiveOptions]=\"{ exact: true }\">Authentication</a>\n    </li>\n    <li *ngIf=\"showModelsOverview\">\n      <a routerLink=\"/models\" routerLinkActive=\"active\" [routerLinkActiveOptions]=\"{ exact: true }\">Models overview</a>\n    </li>\n  </ol>\n\n  <ng-container *ngFor=\"let tag of index\">\n    <header oavOpenable #openable=\"openable\" [open]=\"openTags.has(tag.tag.name)\">\n      <a href=\"javascript:\">\n        <span>{{ tag.tag.name }}</span>\n        <button type=\"button\" class=\"toggle-button ml-auto\" [class.open]=\"openable.open\"></button>\n      </a>\n    </header>\n    <ol [hidden]=\"!openable.open\">\n      <oav-index-nav-endpoints [tag]=\"tag\"></oav-index-nav-endpoints>\n    </ol>\n  </ng-container>\n\n  <ol class=\"oav-bottom-nav\">\n    <li>\n      <span class=\"ml-3\">Powered by</span><br />\n      <a href=\"https://github.com/dhcode/openapi-ui\" target=\"_blank\">OpenAPI UI</a>\n    </li>\n  </ol>\n</nav>\n",
                changeDetection: ChangeDetectionStrategy.OnPush
            }] }
];
/** @nocollapse */
IndexNavComponent.ctorParameters = () => [
    { type: Router },
    { type: OavSettings, decorators: [{ type: Optional }] },
    { type: ChangeDetectorRef }
];
IndexNavComponent.propDecorators = {
    index: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    IndexNavComponent.prototype.openTags;
    /** @type {?} */
    IndexNavComponent.prototype.index;
    /** @type {?} */
    IndexNavComponent.prototype.showHoverLabel;
    /** @type {?} */
    IndexNavComponent.prototype.showAuthentication;
    /** @type {?} */
    IndexNavComponent.prototype.showModelsOverview;
    /**
     * @type {?}
     * @private
     */
    IndexNavComponent.prototype.sub;
    /**
     * @type {?}
     * @private
     */
    IndexNavComponent.prototype.router;
    /**
     * @type {?}
     * @private
     */
    IndexNavComponent.prototype.oavSettings;
    /**
     * @type {?}
     * @private
     */
    IndexNavComponent.prototype.cd;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class OperationViewComponent {
    /**
     * @param {?} route
     * @param {?} openApiService
     */
    constructor(route, openApiService) {
        this.route = route;
        this.openApiService = openApiService;
        this.pathItem = null;
        this.operationItem = null;
        this.tag = null;
        this.error = null;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.sub = combineLatest(this.route.params, this.openApiService.tagIndex).subscribe((/**
         * @param {?} __0
         * @return {?}
         */
        ([params, index]) => {
            this.loadData(index, params.tag, params.opId);
        }));
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.sub) {
            this.sub.unsubscribe();
        }
    }
    /**
     * @param {?} index
     * @param {?} tag
     * @param {?} opId
     * @return {?}
     */
    loadData(index, tag, opId) {
        this.pathItem = null;
        this.operationItem = null;
        this.tag = tag;
        /** @type {?} */
        const tagIndex = index.find((/**
         * @param {?} idx
         * @return {?}
         */
        idx => idx.tag.name === tag));
        if (!tagIndex) {
            this.error = 'noTagFound';
            return;
        }
        for (const pi of tagIndex.paths) {
            /** @type {?} */
            const operationItem = pi.operations.find((/**
             * @param {?} op
             * @return {?}
             */
            op => op.operation.operationId === opId));
            if (operationItem) {
                this.pathItem = pi;
                this.operationItem = operationItem;
            }
        }
        if (!this.operationItem) {
            this.error = 'noOperationFound';
        }
    }
}
OperationViewComponent.decorators = [
    { type: Component, args: [{
                selector: 'oav-operation-view',
                template: "<div class=\"oav-content\">\n  <ng-container *ngIf=\"error === 'noOperationFound'\">\n    Operation was not found\n  </ng-container>\n  <ng-container *ngIf=\"error === 'noTagFound'\">\n    Tag was not found\n  </ng-container>\n  <ng-container *ngIf=\"operationItem\">\n    <oav-operation [tag]=\"tag\" [operationItem]=\"operationItem\" [pathItem]=\"pathItem\"></oav-operation>\n  </ng-container>\n</div>\n"
            }] }
];
/** @nocollapse */
OperationViewComponent.ctorParameters = () => [
    { type: ActivatedRoute },
    { type: OpenapiViewerService }
];
if (false) {
    /**
     * @type {?}
     * @private
     */
    OperationViewComponent.prototype.sub;
    /** @type {?} */
    OperationViewComponent.prototype.pathItem;
    /** @type {?} */
    OperationViewComponent.prototype.operationItem;
    /** @type {?} */
    OperationViewComponent.prototype.tag;
    /** @type {?} */
    OperationViewComponent.prototype.error;
    /**
     * @type {?}
     * @private
     */
    OperationViewComponent.prototype.route;
    /**
     * @type {?}
     * @private
     */
    OperationViewComponent.prototype.openApiService;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class OverviewViewComponent {
    /**
     * @param {?} openApiService
     * @param {?} authService
     * @param {?} cd
     */
    constructor(openApiService, authService, cd) {
        this.openApiService = openApiService;
        this.authService = authService;
        this.cd = cd;
        this.index = [];
        this.spec = null;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.sub = combineLatest([this.openApiService.spec, this.openApiService.tagIndex]).subscribe((/**
         * @param {?} __0
         * @return {?}
         */
        ([spec, index]) => {
            this.spec = spec;
            this.index = index;
            this.cd.detectChanges();
        }));
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.sub.unsubscribe();
    }
}
OverviewViewComponent.decorators = [
    { type: Component, args: [{
                selector: 'oav-overview-view',
                template: "<div class=\"oav-content\">\n  <h1>{{ spec.info.title }}</h1>\n\n  <p *ngIf=\"spec.info.description\">\n    <markdown [data]=\"spec.info.description\"></markdown>\n  </p>\n\n  <div *ngIf=\"spec.info.termsOfService\">\n    <a [href]=\"spec.info.termsOfService\" target=\"_blank\">Terms of service</a>\n  </div>\n  <div *ngIf=\"spec.info.contact?.url\">\n    <a [href]=\"spec.info.contact.url\" target=\"_blank\">{{ spec.info.contact.name || 'Contact link' }}</a>\n  </div>\n  <div *ngIf=\"spec.info.contact?.email\">\n    <a [href]=\"'mailto:' + spec.info.contact.email\" target=\"_blank\">{{ spec.info.contact.name || 'Contact email' }}</a>\n  </div>\n  <div *ngIf=\"spec.info.license?.url\">\n    <a [href]=\"spec.info.license?.url\" target=\"_blank\">License: {{ spec.info.license.name }}</a>\n  </div>\n  <div *ngIf=\"spec.info.license && !spec.info.license.url\">License: {{ spec.info.license.name }}</div>\n\n  <ng-container *ngFor=\"let tag of index\">\n    <h2>{{ tag.tag.name }}</h2>\n    <p *ngIf=\"tag.tag.description\">{{ tag.tag.description }}</p>\n    <small *ngIf=\"tag.tag.externalDocs\" class=\"extra-link\">\n      {{ tag.tag.externalDocs.description }}:\n      <a [href]=\"tag.tag.externalDocs.url\" target=\"_blank\">{{ tag.tag.externalDocs.url }}</a>\n    </small>\n\n    <oav-overview-view-endpoints [tag]=\"tag\"></oav-overview-view-endpoints>\n  </ng-container>\n</div>\n",
                changeDetection: ChangeDetectionStrategy.OnPush
            }] }
];
/** @nocollapse */
OverviewViewComponent.ctorParameters = () => [
    { type: OpenapiViewerService },
    { type: OpenapiAuthService },
    { type: ChangeDetectorRef }
];
if (false) {
    /** @type {?} */
    OverviewViewComponent.prototype.index;
    /** @type {?} */
    OverviewViewComponent.prototype.spec;
    /**
     * @type {?}
     * @private
     */
    OverviewViewComponent.prototype.sub;
    /**
     * @type {?}
     * @private
     */
    OverviewViewComponent.prototype.openApiService;
    /**
     * @type {?}
     * @private
     */
    OverviewViewComponent.prototype.authService;
    /**
     * @type {?}
     * @private
     */
    OverviewViewComponent.prototype.cd;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AuthenticationViewComponent {
    /**
     * @param {?} authService
     * @param {?} route
     */
    constructor(authService, route) {
        this.authService = authService;
        this.route = route;
        this.schemes = [];
        this.sub = null;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.sub = this.authService.securitySchemes.subscribe((/**
         * @param {?} schemas
         * @return {?}
         */
        schemas => (this.schemes = schemas)));
        this.route.fragment.subscribe((/**
         * @param {?} fragment
         * @return {?}
         */
        fragment => {
            if (fragment && fragment.length) {
                /** @type {?} */
                const params = parseQueryString(fragment);
                this.authService.handleOAuthCallback(params);
            }
        }));
        this.route.queryParams.subscribe((/**
         * @param {?} params
         * @return {?}
         */
        params => {
            if (params.state) {
                this.authService.handleOAuthCallback(params);
            }
        }));
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.sub) {
            this.sub.unsubscribe();
        }
    }
}
AuthenticationViewComponent.decorators = [
    { type: Component, args: [{
                selector: 'oav-authentication-view',
                template: "<div class=\"oav-content\">\n  <h1>Authentication schemes</h1>\n\n  <div *ngFor=\"let scheme of schemes\">\n    <h3>\n      {{ scheme.name }}\n      <span class=\"oav-auth-status ml-1\" [class.required]=\"!scheme.authenticated\" [class.ok]=\"scheme.authenticated\"></span>\n      <span class=\"ml-1 normal-text\">({{ scheme.securityScheme.type }})</span>\n    </h3>\n\n    <markdown *ngIf=\"scheme.securityScheme.description\" [data]=\"scheme.securityScheme.description\"></markdown>\n\n    <oav-auth-credentials [securityScheme]=\"scheme\"></oav-auth-credentials>\n  </div>\n\n  <p *ngIf=\"!schemes.length\">\n    No authentication schemes available.\n  </p>\n</div>\n"
            }] }
];
/** @nocollapse */
AuthenticationViewComponent.ctorParameters = () => [
    { type: OpenapiAuthService },
    { type: ActivatedRoute }
];
if (false) {
    /** @type {?} */
    AuthenticationViewComponent.prototype.schemes;
    /**
     * @type {?}
     * @private
     */
    AuthenticationViewComponent.prototype.sub;
    /**
     * @type {?}
     * @private
     */
    AuthenticationViewComponent.prototype.authService;
    /**
     * @type {?}
     * @private
     */
    AuthenticationViewComponent.prototype.route;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @record
 */
function DefinitionCategory() { }
if (false) {
    /** @type {?} */
    DefinitionCategory.prototype.name;
    /** @type {?} */
    DefinitionCategory.prototype.label;
    /** @type {?} */
    DefinitionCategory.prototype.definitions;
}
/**
 * @record
 */
function DefinitionItem() { }
if (false) {
    /** @type {?} */
    DefinitionItem.prototype.name;
    /** @type {?|undefined} */
    DefinitionItem.prototype.schema;
    /** @type {?|undefined} */
    DefinitionItem.prototype.response;
    /** @type {?|undefined} */
    DefinitionItem.prototype.parameter;
    /** @type {?|undefined} */
    DefinitionItem.prototype.example;
    /** @type {?|undefined} */
    DefinitionItem.prototype.requestBody;
    /** @type {?|undefined} */
    DefinitionItem.prototype.header;
    /** @type {?|undefined} */
    DefinitionItem.prototype.securityScheme;
    /** @type {?|undefined} */
    DefinitionItem.prototype.link;
    /** @type {?|undefined} */
    DefinitionItem.prototype.callback;
}
/** @type {?} */
const categoryLabels = {
    schema: 'Schemas',
    response: 'Responses',
    parameter: 'Parameters',
    example: 'Examples',
    requestBody: 'RequestBodies',
    header: 'Headers',
    securityScheme: 'SecuritySchemes',
    link: 'Links',
    callback: 'Callbacks'
};
class ModelsViewComponent {
    /**
     * @param {?} openApiService
     * @param {?} oavSettings
     * @param {?} cd
     */
    constructor(openApiService, oavSettings, cd) {
        this.openApiService = openApiService;
        this.oavSettings = oavSettings;
        this.cd = cd;
        this.categories = [];
        if (!this.oavSettings) {
            this.oavSettings = defaultOavSettings;
        }
    }
    /**
     * @return {?}
     */
    get showRaw() {
        return this.oavSettings.showRawModelDefinition;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.sub = this.openApiService.spec.subscribe((/**
         * @param {?} spec
         * @return {?}
         */
        spec => {
            this.categories = identifyModels(spec);
            this.cd.detectChanges();
        }));
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.sub.unsubscribe();
    }
}
ModelsViewComponent.decorators = [
    { type: Component, args: [{
                selector: 'oav-models-view',
                template: "<div class=\"oav-content\">\n  <h1>Definitions</h1>\n\n  <ng-container *ngFor=\"let cat of categories\">\n    <h2>{{ cat.label }}</h2>\n\n    <div *ngFor=\"let def of cat.definitions\">\n      <h3>{{ def.name }}</h3>\n      <div class=\"ml-3\" *ngIf=\"def.schema\">\n        <oav-json-schema [schema]=\"def.schema\" [defaultModelName]=\"def.name\"></oav-json-schema>\n\n        <ng-container *ngIf=\"showRaw\">\n          <h4 oavOpenable #rawOpenable=\"openable\" class=\"toggle-button\">Raw {{ def.name }} model definition</h4>\n          <oav-code-input *ngIf=\"rawOpenable.open\" mode=\"json\" [ngModel]=\"def.schema | json\" readonly></oav-code-input>\n        </ng-container>\n      </div>\n    </div>\n  </ng-container>\n</div>\n",
                changeDetection: ChangeDetectionStrategy.OnPush
            }] }
];
/** @nocollapse */
ModelsViewComponent.ctorParameters = () => [
    { type: OpenapiViewerService },
    { type: OavSettings, decorators: [{ type: Optional }] },
    { type: ChangeDetectorRef }
];
if (false) {
    /** @type {?} */
    ModelsViewComponent.prototype.categories;
    /**
     * @type {?}
     * @private
     */
    ModelsViewComponent.prototype.sub;
    /**
     * @type {?}
     * @private
     */
    ModelsViewComponent.prototype.openApiService;
    /**
     * @type {?}
     * @private
     */
    ModelsViewComponent.prototype.oavSettings;
    /**
     * @type {?}
     * @private
     */
    ModelsViewComponent.prototype.cd;
}
/**
 * @param {?} propName
 * @param {?} categories
 * @return {?}
 */
function getCategory(propName, categories) {
    /** @type {?} */
    const existing = categories.find((/**
     * @param {?} c
     * @return {?}
     */
    c => c.name === propName));
    if (existing) {
        return existing;
    }
    /** @type {?} */
    const cat = {
        name: propName,
        label: categoryLabels[propName],
        definitions: []
    };
    categories.push(cat);
    return cat;
}
/**
 * @param {?} defItem
 * @param {?} categories
 * @return {?}
 */
function addToCategory(defItem, categories) {
    /** @type {?} */
    const categoryPropsNames = Object.keys(categoryLabels);
    for (const propName of categoryPropsNames) {
        if (defItem[propName]) {
            /** @type {?} */
            const cat = getCategory(propName, categories);
            cat.definitions.push(defItem);
        }
    }
}
/**
 * @param {?} spec
 * @return {?}
 */
function identifyModels(spec) {
    /** @type {?} */
    const categories = [];
    // OASv2
    if (spec.definitions) {
        Object.keys(spec.definitions)
            .map((/**
         * @param {?} name
         * @return {?}
         */
        name => getDefinitionsItem('schema', name, spec.definitions[name])))
            .forEach((/**
         * @param {?} defItem
         * @return {?}
         */
        defItem => addToCategory(defItem, categories)));
    }
    if (spec.responses) {
        Object.keys(spec.responses)
            .map((/**
         * @param {?} name
         * @return {?}
         */
        name => getDefinitionsItem('response', name, spec.definitions[name])))
            .forEach((/**
         * @param {?} defItem
         * @return {?}
         */
        defItem => addToCategory(defItem, categories)));
    }
    if (spec.parameters) {
        Object.keys(spec.parameters)
            .map((/**
         * @param {?} name
         * @return {?}
         */
        name => getDefinitionsItem('parameter', name, spec.definitions[name])))
            .forEach((/**
         * @param {?} defItem
         * @return {?}
         */
        defItem => addToCategory(defItem, categories)));
    }
    // OASv3
    if (spec.components) {
        Object.keys(spec.components).forEach((/**
         * @param {?} categoryPlural
         * @return {?}
         */
        categoryPlural => {
            /** @type {?} */
            const category = (/** @type {?} */ (categoryPlural.substr(0, categoryPlural.length - 1)));
            /** @type {?} */
            const cat = getCategory(category, categories);
            Object.keys(spec.components[categoryPlural])
                .map((/**
             * @param {?} name
             * @return {?}
             */
            name => getDefinitionsItem(category, name, spec.components[categoryPlural][name])))
                .forEach((/**
             * @param {?} defItem
             * @return {?}
             */
            defItem => {
                cat.definitions.push(defItem);
            }));
        }));
    }
    return categories;
}
/**
 * @param {?} category
 * @param {?} name
 * @param {?} component
 * @return {?}
 */
function getDefinitionsItem(category, name, component) {
    /** @type {?} */
    const item = { name };
    item[category] = component;
    return item;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const routes = [
    {
        path: '',
        pathMatch: 'full',
        component: OverviewViewComponent
    },
    {
        path: 'auth',
        component: AuthenticationViewComponent
    },
    {
        path: 'models',
        component: ModelsViewComponent
    },
    {
        path: ':tag/:opId',
        component: OperationViewComponent
    }
];
class OpenapiViewerRoutingModule {
}
OpenapiViewerRoutingModule.decorators = [
    { type: NgModule, args: [{
                imports: [RouterModule.forRoot(routes, { useHash: true })],
                exports: [RouterModule]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TabNavItemDirective {
    /**
     * @param {?} templateRef
     */
    constructor(templateRef) {
        this.templateRef = templateRef;
        this.id = randomHex(8);
        this.active = false;
    }
    /**
     * @param {?} label
     * @return {?}
     */
    set oavTabNavItem(label) {
        this.label = label;
    }
    /**
     * @param {?} id
     * @return {?}
     */
    set oavTabNavItemId(id) {
        this.id = id;
    }
    /**
     * @param {?} details
     * @return {?}
     */
    set oavTabNavItemDetails(details) {
        this.details = details;
    }
}
TabNavItemDirective.decorators = [
    { type: Directive, args: [{
                selector: '[oavTabNavItem]'
            },] }
];
/** @nocollapse */
TabNavItemDirective.ctorParameters = () => [
    { type: TemplateRef }
];
TabNavItemDirective.propDecorators = {
    oavTabNavItem: [{ type: Input }],
    oavTabNavItemId: [{ type: Input }],
    oavTabNavItemDetails: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    TabNavItemDirective.prototype.id;
    /** @type {?} */
    TabNavItemDirective.prototype.label;
    /** @type {?} */
    TabNavItemDirective.prototype.details;
    /** @type {?} */
    TabNavItemDirective.prototype.active;
    /** @type {?} */
    TabNavItemDirective.prototype.templateRef;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TabNavComponent {
    constructor() {
        this.tabs = [];
        this.tabChange = new EventEmitter();
        this.destroy = new Subject();
    }
    /**
     * @param {?} tabs
     * @return {?}
     */
    set tabList(tabs) {
        if (tabs) {
            this.tabs = tabs.toArray();
            this.checkForActiveTab();
        }
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (changes.tab && !changes.tab.firstChange) {
            this.showTab(this.tab);
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.destroy.complete();
    }
    /**
     * @private
     * @return {?}
     */
    checkForActiveTab() {
        if (!this.tabs.find((/**
         * @param {?} t
         * @return {?}
         */
        t => t.active)) && this.tabs.length) {
            if (this.tab && this.getTabById(this.tab)) {
                this.showTab(this.tab);
            }
            else {
                this.showTab(this.tabs[0].id);
            }
        }
    }
    /**
     * @private
     * @param {?} id
     * @return {?}
     */
    getTabById(id) {
        return this.tabs.find((/**
         * @param {?} tab
         * @return {?}
         */
        tab => tab.id === id));
    }
    /**
     * @param {?} id
     * @param {?=} notify
     * @return {?}
     */
    showTab(id, notify = false) {
        for (const tab of this.tabs) {
            if (tab.active && tab.id !== id) {
                tab.active = false;
            }
            if (!tab.active && tab.id === id) {
                tab.active = true;
                this.tab = tab.id;
                this.tabContent = tab.templateRef;
                if (notify) {
                    this.tabChange.next(tab.id);
                }
            }
        }
    }
}
TabNavComponent.decorators = [
    { type: Component, args: [{
                selector: 'oav-tab-nav',
                template: "<nav class=\"oav-tab-nav\">\n  <a href=\"javascript:\" *ngFor=\"let tab of tabs\" [class.active]=\"tab.active\" (click)=\"showTab(tab.id, true)\">\n    <oav-template-outlet [tpl]=\"tab.label\" [ctx]=\"tab\"></oav-template-outlet>\n  </a>\n</nav>\n<div class=\"oav-tab-content\">\n  <ng-container *ngTemplateOutlet=\"tabContent\"></ng-container>\n</div>\n"
            }] }
];
/** @nocollapse */
TabNavComponent.ctorParameters = () => [];
TabNavComponent.propDecorators = {
    tabChange: [{ type: Output }],
    tab: [{ type: Input }],
    tabList: [{ type: ContentChildren, args: [TabNavItemDirective,] }]
};
if (false) {
    /** @type {?} */
    TabNavComponent.prototype.tabs;
    /** @type {?} */
    TabNavComponent.prototype.tabChange;
    /** @type {?} */
    TabNavComponent.prototype.tab;
    /** @type {?} */
    TabNavComponent.prototype.tabContent;
    /**
     * @type {?}
     * @private
     */
    TabNavComponent.prototype.destroy;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TemplateOutletComponent {
    /**
     * @return {?}
     */
    isTemplate() {
        return this.tpl instanceof TemplateRef;
    }
}
TemplateOutletComponent.decorators = [
    { type: Component, args: [{
                selector: 'oav-template-outlet',
                template: "<ng-container *ngIf=\"isTemplate(); else noTpl\">\n  <ng-container *ngTemplateOutlet=\"tpl; context: ctx\"></ng-container>\n</ng-container>\n<ng-template #noTpl>{{ tpl }}</ng-template>\n",
                changeDetection: ChangeDetectionStrategy.OnPush
            }] }
];
TemplateOutletComponent.propDecorators = {
    tpl: [{ type: Input }],
    ctx: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    TemplateOutletComponent.prototype.tpl;
    /** @type {?} */
    TemplateOutletComponent.prototype.ctx;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class RequestBodyInputComponent {
    constructor() {
        this.formGroup = new FormGroup({
            contentType: new FormControl()
        });
        this.contentParameters = {};
        this.contentTypes = [];
        this.currentParameters = [];
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.addToFormGroup.addControl('requestBody', this.formGroup);
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.addToFormGroup.removeControl('requestBody');
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        this.identifyParameters();
    }
    /**
     * @private
     * @return {?}
     */
    identifyParameters() {
        this.contentParameters = {};
        this.contentTypes = [];
        if (!this.requestBody || !this.requestBody.content) {
            return;
        }
        this.contentTypes = Object.keys(this.requestBody.content);
        if (this.contentTypes.length) {
            for (const type of this.contentTypes) {
                this.contentParameters[type] = this.contentToParameter(type, this.requestBody.content[type]);
            }
            this.currentContentType = this.contentTypes[0];
            this.currentParameters = this.contentParameters[this.currentContentType];
            this.formGroup.patchValue({ contentType: this.currentContentType });
        }
    }
    /**
     * @private
     * @param {?} mediaType
     * @param {?} def
     * @return {?}
     */
    contentToParameter(mediaType, def) {
        if ((mediaType === 'application/x-www-form-urlencoded' || mediaType.startsWith('multipart')) && def.schema) {
            /** @type {?} */
            const schema = (/** @type {?} */ (def.schema));
            if ((schema.type === 'object' || !schema.type) && schema.properties) {
                return Object.keys(schema.properties).map((/**
                 * @param {?} key
                 * @return {?}
                 */
                key => {
                    /** @type {?} */
                    const propScheme = (/** @type {?} */ (schema.properties[key]));
                    return {
                        name: key,
                        customIn: 'form-data',
                        required: schema.required && schema.required.includes(key),
                        description: propScheme.description,
                        schema: propScheme
                    };
                }));
            }
        }
        /** @type {?} */
        const param = {
            name: 'requestBody',
            customIn: 'body',
            required: this.requestBody.required,
            description: this.requestBody.description,
            schema: def.schema,
            example: def.example,
            examples: def.examples
        };
        return [param];
    }
}
RequestBodyInputComponent.decorators = [
    { type: Component, args: [{
                selector: 'oav-request-body-input',
                template: "<ng-container *ngFor=\"let param of currentParameters\">\n  <oav-parameter [parameter]=\"param\" [formGroup]=\"formGroup\"></oav-parameter>\n</ng-container>\n"
            }] }
];
/** @nocollapse */
RequestBodyInputComponent.ctorParameters = () => [];
RequestBodyInputComponent.propDecorators = {
    requestBody: [{ type: Input }],
    addToFormGroup: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    RequestBodyInputComponent.prototype.requestBody;
    /** @type {?} */
    RequestBodyInputComponent.prototype.addToFormGroup;
    /** @type {?} */
    RequestBodyInputComponent.prototype.formGroup;
    /** @type {?} */
    RequestBodyInputComponent.prototype.contentParameters;
    /** @type {?} */
    RequestBodyInputComponent.prototype.contentTypes;
    /** @type {?} */
    RequestBodyInputComponent.prototype.currentContentType;
    /** @type {?} */
    RequestBodyInputComponent.prototype.currentParameters;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AuthCredentialsComponent {
    /**
     * @param {?} authService
     */
    constructor(authService) {
        this.authService = authService;
        this.displayMode = 'unknown';
        this.username = new FormControl('');
        this.password = new FormControl('');
        this.apiKey = new FormControl('');
        this.clientId = new FormControl('');
        this.clientSecret = new FormControl('');
        this.scopes = new FormArray([]);
        this.flow = new FormControl('');
        this.remember = new FormControl(false);
        this.scopesInfo = [];
        this.loading = false;
        this.error = null;
        /**
         * The available flows
         */
        this.flows = [];
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.sub = this.flow.valueChanges.subscribe((/**
         * @param {?} flow
         * @return {?}
         */
        flow => {
            this.readOAuthCredentials(flow);
        }));
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (changes.securityScheme && this.securityScheme) {
            this.checkScheme();
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.sub.unsubscribe();
    }
    /**
     * @return {?}
     */
    checkScheme() {
        /** @type {?} */
        const type = this.securityScheme.securityScheme.type;
        if (type === 'apiKey') {
            this.displayMode = 'apiKey';
            this.readApiKey();
        }
        else if (type === 'http' || type === 'basic') {
            this.displayMode = 'usernamePassword';
            this.readHttpCredentials();
        }
        else if (type === 'oauth2' || type === 'openIdConnect') {
            this.displayMode = 'oauth';
            this.readOAuthCredentials();
        }
        else {
            this.displayMode = 'unknown';
        }
        if (this.securityScheme.remember) {
            this.remember.patchValue(true);
        }
    }
    /**
     * @return {?}
     */
    readApiKey() {
        this.formGroup = new FormGroup({ apiKey: this.apiKey, remember: this.remember });
        if (typeof this.securityScheme.credentials === 'string') {
            this.apiKey.patchValue(this.securityScheme.credentials);
        }
        else {
            this.apiKey.patchValue('');
        }
    }
    /**
     * @return {?}
     */
    readHttpCredentials() {
        this.formGroup = new FormGroup({ username: this.username, password: this.password, remember: this.remember });
        /** @type {?} */
        const credentials = (/** @type {?} */ (this.securityScheme.credentials));
        if (typeof credentials === 'object' && credentials) {
            if (typeof credentials.username === 'string') {
                this.username.patchValue(credentials.username);
            }
            else {
                this.username.patchValue('');
            }
            if (typeof credentials.password === 'string') {
                this.password.patchValue(credentials.password);
            }
            else {
                this.password.patchValue('');
            }
        }
        else {
            this.username.patchValue('');
            this.password.patchValue('');
        }
    }
    /**
     * @param {?=} flow
     * @return {?}
     */
    readOAuthCredentials(flow) {
        /** @type {?} */
        const credentials = (/** @type {?} */ (this.securityScheme.credentials));
        if (!flow && credentials && credentials.flow) {
            flow = credentials.flow;
        }
        this.flows = identifyFlows(this.securityScheme.securityScheme);
        /** @type {?} */
        const currentFlow = flow ? this.flows.find((/**
         * @param {?} f
         * @return {?}
         */
        f => f.flow === flow)) : this.flows[0];
        if (!currentFlow) {
            return;
        }
        if (credentials) {
            this.token = credentials.token;
        }
        this.currentFlow = currentFlow;
        this.scopesInfo = currentFlow.scopes;
        this.formGroup = new FormGroup({
            clientId: this.clientId,
            clientSecret: this.clientSecret,
            username: this.username,
            password: this.password,
            scopes: this.scopes,
            remember: this.remember
        });
        this.scopes.clear();
        this.remember.patchValue(true);
        this.flow.patchValue(currentFlow.flow);
        if (typeof credentials === 'object' && credentials) {
            this.clientId.patchValue(credentials.clientId);
            this.clientSecret.patchValue(credentials.clientSecret);
            this.username.patchValue(credentials.username || '');
            this.password.patchValue(credentials.password || '');
            for (const scope of this.scopesInfo) {
                this.scopes.push(new FormControl(credentials.scopes.includes(scope.scope)));
            }
        }
        else {
            this.clientId.patchValue('');
            for (const scope of this.scopesInfo) {
                this.scopes.push(new FormControl(true));
            }
        }
    }
    /**
     * @return {?}
     */
    save() {
        if (this.displayMode === 'apiKey') {
            this.authService.updateCredentials(this.securityScheme.name, this.apiKey.value, this.apiKey.value.length > 0, this.remember.value);
        }
        if (this.displayMode === 'usernamePassword') {
            this.authService.updateCredentials(this.securityScheme.name, {
                username: this.username.value,
                password: this.password.value
            }, this.username.value.length > 0, this.remember.value);
        }
        if (this.displayMode === 'oauth') {
            /** @type {?} */
            const scopes = this.scopesInfo.map((/**
             * @param {?} s
             * @param {?} i
             * @return {?}
             */
            (s, i) => (this.scopes.value[i] ? s.scope : null))).filter((/**
             * @param {?} s
             * @return {?}
             */
            s => !!s));
            /** @type {?} */
            const credentials = {
                clientId: this.clientId.value,
                clientSecret: this.clientSecret.value,
                username: this.username.value,
                password: this.password.value,
                flow: this.flow.value,
                scopes,
                token: null,
                nonce: randomHex(16),
                redirectUri: getCurrentUrl()
            };
            this.loading = true;
            this.error = null;
            this.authService.runOAuthAuthorization(this.securityScheme.name, credentials).subscribe((/**
             * @return {?}
             */
            () => {
                this.loading = false;
            }), (/**
             * @param {?} err
             * @return {?}
             */
            err => {
                this.loading = false;
                this.error = err;
            }));
        }
        this.formGroup.markAsPristine();
    }
    /**
     * @return {?}
     */
    removeToken() {
        this.authService.removeToken(this.securityScheme.name);
    }
}
AuthCredentialsComponent.decorators = [
    { type: Component, args: [{
                selector: 'oav-auth-credentials',
                template: "<form action=\"javascript:\" (submit)=\"save()\" *ngIf=\"formGroup\" [formGroup]=\"formGroup\">\n  <ng-container *ngIf=\"displayMode === 'apiKey'\">\n    <div class=\"oav-parameter\">\n      <div class=\"name\">\n        <div><strong>API Key</strong>:</div>\n        <div class=\"param-in\">\n          {{ securityScheme.securityScheme.in }}\n          <span class=\"format\" *ngIf=\"securityScheme.securityScheme.scheme\">({{ securityScheme.securityScheme.scheme }})</span>\n        </div>\n      </div>\n      <div class=\"input\">\n        <input [formControl]=\"apiKey\" name=\"apiKey\" />\n      </div>\n    </div>\n    <button class=\"send-btn\" [disabled]=\"formGroup.pristine\">Save</button>\n    <label class=\"ml-2\">\n      <input type=\"checkbox\" [formControl]=\"remember\" />\n      Remember in browser session\n    </label>\n  </ng-container>\n  <ng-container *ngIf=\"displayMode === 'usernamePassword'\">\n    <div class=\"oav-parameter\">\n      <div class=\"name\">\n        <div><strong>Username</strong>:</div>\n        <div class=\"param-in\">\n          {{ securityScheme.securityScheme.in }}\n          <span class=\"format\" *ngIf=\"securityScheme.securityScheme.scheme\">({{ securityScheme.securityScheme.scheme }})</span>\n        </div>\n      </div>\n      <div class=\"input\">\n        <input [formControl]=\"username\" name=\"username\" />\n      </div>\n    </div>\n    <div class=\"oav-parameter\">\n      <div class=\"name\">\n        <div><strong>Password</strong>:</div>\n        <div class=\"param-in\">\n          {{ securityScheme.securityScheme.in }}\n          <span class=\"format\" *ngIf=\"securityScheme.securityScheme.scheme\">({{ securityScheme.securityScheme.scheme }})</span>\n        </div>\n      </div>\n      <div class=\"input\">\n        <input [formControl]=\"password\" type=\"password\" name=\"password\" />\n      </div>\n    </div>\n    <button class=\"send-btn\" [disabled]=\"formGroup.pristine\">Save</button>\n    <label class=\"ml-2\">\n      <input type=\"checkbox\" [formControl]=\"remember\" />\n      Remember in browser session\n    </label>\n  </ng-container>\n  <ng-container *ngIf=\"displayMode === 'oauth'\">\n    <div class=\"oav-parameter\" *ngIf=\"token\">\n      <div class=\"name\">\n        <div><strong>Current token</strong>:</div>\n      </div>\n      <div class=\"input\">\n        <div>\n          Access token: <code>{{ token.access_token }}</code>\n        </div>\n        <div>\n          Token type: <code>{{ token.token_type }}</code>\n        </div>\n        <div *ngIf=\"token.expires_at\">Expires {{ token.expires_at | date: 'long' }}</div>\n        <button type=\"button\" class=\"dismiss-btn\" (click)=\"removeToken()\">Remove token</button>\n      </div>\n    </div>\n    <div class=\"oav-parameter\">\n      <div class=\"name\">\n        <div><strong>Flow</strong>:</div>\n      </div>\n      <div class=\"input\">\n        <select [formControl]=\"flow\" name=\"flow\">\n          <option *ngFor=\"let f of flows\" [value]=\"f.flow\">{{ f.flow }}</option>\n        </select>\n      </div>\n    </div>\n    <div class=\"oav-parameter\">\n      <div class=\"name\">\n        <div><strong>Client ID</strong>:</div>\n      </div>\n      <div class=\"input\">\n        <input [formControl]=\"clientId\" name=\"clientId\" />\n      </div>\n    </div>\n    <div class=\"oav-parameter\" *ngIf=\"flow.value === 'authorizationCode' || flow.value === 'clientCredentials'\">\n      <div class=\"name\">\n        <div><strong>Client Secret</strong>:</div>\n      </div>\n      <div class=\"input\">\n        <input [formControl]=\"clientSecret\" type=\"password\" name=\"clientSecret\" />\n      </div>\n    </div>\n    <div class=\"oav-parameter\" *ngIf=\"flow.value === 'password'\">\n      <div class=\"name\">\n        <div><strong>Username</strong>:</div>\n      </div>\n      <div class=\"input\">\n        <input [formControl]=\"username\" name=\"username\" />\n      </div>\n    </div>\n    <div class=\"oav-parameter\" *ngIf=\"flow.value === 'password'\">\n      <div class=\"name\">\n        <div><strong>Password</strong>:</div>\n      </div>\n      <div class=\"input\">\n        <input [formControl]=\"password\" type=\"password\" name=\"password\" />\n      </div>\n    </div>\n    <div class=\"oav-parameter\">\n      <div class=\"name\">\n        <div><strong>Scopes</strong>:</div>\n        <div class=\"param-in\">\n          {{ securityScheme.securityScheme.in }}\n          <span class=\"format\" *ngIf=\"securityScheme.securityScheme.scheme\">({{ securityScheme.securityScheme.scheme }})</span>\n        </div>\n      </div>\n      <div class=\"input\" [formArrayName]=\"'scopes'\">\n        <div *ngFor=\"let scope of scopesInfo; let i = index\">\n          <label>\n            <input type=\"checkbox\" [formControlName]=\"i\" [value]=\"scope.scope\" />\n            {{ scope.description }}\n          </label>\n        </div>\n      </div>\n    </div>\n    <button class=\"send-btn\" [disabled]=\"formGroup.invalid || loading\" [class.loading]=\"loading\">Authenticate</button>\n    <label class=\"ml-2\">\n      <input type=\"checkbox\" [formControl]=\"remember\" required />\n      Remember in browser session\n    </label>\n    <div class=\"oav-error\" *ngIf=\"error\">{{ error }}</div>\n  </ng-container>\n</form>\n<ng-container *ngIf=\"displayMode === 'unknown'\">\n  <p>Unsupported authentication scheme '{{ securityScheme?.securityScheme?.type }}'</p>\n</ng-container>\n"
            }] }
];
/** @nocollapse */
AuthCredentialsComponent.ctorParameters = () => [
    { type: OpenapiAuthService }
];
AuthCredentialsComponent.propDecorators = {
    securityScheme: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    AuthCredentialsComponent.prototype.displayMode;
    /** @type {?} */
    AuthCredentialsComponent.prototype.username;
    /** @type {?} */
    AuthCredentialsComponent.prototype.password;
    /** @type {?} */
    AuthCredentialsComponent.prototype.apiKey;
    /** @type {?} */
    AuthCredentialsComponent.prototype.clientId;
    /** @type {?} */
    AuthCredentialsComponent.prototype.clientSecret;
    /** @type {?} */
    AuthCredentialsComponent.prototype.scopes;
    /** @type {?} */
    AuthCredentialsComponent.prototype.flow;
    /** @type {?} */
    AuthCredentialsComponent.prototype.remember;
    /** @type {?} */
    AuthCredentialsComponent.prototype.scopesInfo;
    /** @type {?} */
    AuthCredentialsComponent.prototype.loading;
    /** @type {?} */
    AuthCredentialsComponent.prototype.error;
    /** @type {?} */
    AuthCredentialsComponent.prototype.token;
    /**
     * The available flows
     * @type {?}
     */
    AuthCredentialsComponent.prototype.flows;
    /** @type {?} */
    AuthCredentialsComponent.prototype.currentFlow;
    /** @type {?} */
    AuthCredentialsComponent.prototype.formGroup;
    /** @type {?} */
    AuthCredentialsComponent.prototype.securityScheme;
    /**
     * @type {?}
     * @private
     */
    AuthCredentialsComponent.prototype.sub;
    /**
     * @type {?}
     * @private
     */
    AuthCredentialsComponent.prototype.authService;
}
/**
 * @return {?}
 */
function getCurrentUrl() {
    /** @type {?} */
    const url = window.location.href;
    /** @type {?} */
    const idx = url.indexOf('?');
    if (idx !== -1) {
        return url.slice(0, idx);
    }
    return url;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AuthStatusComponent {
    /**
     * @param {?} oavSettings
     * @param {?} authService
     * @param {?} cd
     */
    constructor(oavSettings, authService, cd) {
        this.oavSettings = oavSettings;
        this.authService = authService;
        this.cd = cd;
        this.schemeNames = [];
        if (!this.oavSettings) {
            this.oavSettings = defaultOavSettings;
        }
        this.showAuthentication = this.oavSettings.enableAuthentication;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.sub = this.authService.updatedCredentials.subscribe((/**
         * @param {?} schemeName
         * @return {?}
         */
        schemeName => {
            if (this.schemeNames.includes(schemeName)) {
                this.cd.detectChanges();
            }
        }));
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.sub.unsubscribe();
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (changes.operation) {
            this.authStatus = this.authService.getAuthStatus(this.operation.security);
            this.requiredSchemes = this.authService.getRequiredSchemes(this.operation.security);
            this.schemeNames = [];
            for (const reqs of this.requiredSchemes) {
                for (const scheme of reqs) {
                    this.schemeNames.push(scheme.name);
                }
            }
        }
    }
}
AuthStatusComponent.decorators = [
    { type: Component, args: [{
                selector: 'oav-auth-status',
                template: "<span\n  #statusIcon\n  class=\"oav-auth-status ml-1\"\n  *ngIf=\"showAuthentication\"\n  [class.required]=\"authStatus === 'required'\"\n  [class.ok]=\"authStatus === 'ok'\"\n  [oavTooltip]=\"authStatus !== 'none' ? portal : null\"\n></span>\n\n<oav-tooltip *cdkPortal>\n  <ng-container *ngFor=\"let reqs of requiredSchemes\">\n    <div *ngFor=\"let status of reqs\">\n      <h4 class=\"m-0\">\n        {{ status.name }}\n        <span class=\"oav-auth-status ml-1\" [class.required]=\"!status.authenticated\" [class.ok]=\"status.authenticated\"></span>\n        <span class=\"ml-1 normal-text\" *ngIf=\"status.authenticated\">authenticated</span>\n        <span class=\"ml-1 normal-text\" *ngIf=\"!status.authenticated\">not provided</span>\n      </h4>\n      <ng-container *ngIf=\"status.scopes?.length\"> Required scopes: {{ status.scopes.join(', ') }} </ng-container>\n    </div>\n  </ng-container>\n</oav-tooltip>\n",
                changeDetection: ChangeDetectionStrategy.OnPush
            }] }
];
/** @nocollapse */
AuthStatusComponent.ctorParameters = () => [
    { type: OavSettings, decorators: [{ type: Optional }] },
    { type: OpenapiAuthService },
    { type: ChangeDetectorRef }
];
AuthStatusComponent.propDecorators = {
    operation: [{ type: Input }],
    portal: [{ type: ViewChild, args: [CdkPortal, { static: true },] }],
    statusIcon: [{ type: ViewChild, args: ['statusIcon', { static: false },] }]
};
if (false) {
    /** @type {?} */
    AuthStatusComponent.prototype.operation;
    /** @type {?} */
    AuthStatusComponent.prototype.portal;
    /** @type {?} */
    AuthStatusComponent.prototype.statusIcon;
    /** @type {?} */
    AuthStatusComponent.prototype.authStatus;
    /** @type {?} */
    AuthStatusComponent.prototype.showAuthentication;
    /** @type {?} */
    AuthStatusComponent.prototype.requiredSchemes;
    /** @type {?} */
    AuthStatusComponent.prototype.schemeNames;
    /**
     * @type {?}
     * @private
     */
    AuthStatusComponent.prototype.sub;
    /**
     * @type {?}
     * @private
     */
    AuthStatusComponent.prototype.oavSettings;
    /**
     * @type {?}
     * @private
     */
    AuthStatusComponent.prototype.authService;
    /**
     * @type {?}
     * @private
     */
    AuthStatusComponent.prototype.cd;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TooltipComponent {
    constructor() { }
    /**
     * @return {?}
     */
    ngOnInit() { }
}
TooltipComponent.decorators = [
    { type: Component, args: [{
                selector: 'oav-tooltip',
                template: "<ng-content></ng-content>\n"
            }] }
];
/** @nocollapse */
TooltipComponent.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TooltipDirective {
    /**
     * @param {?} overlay
     * @param {?} elementRef
     * @param {?} viewContainer
     */
    constructor(overlay, elementRef, viewContainer) {
        this.overlay = overlay;
        this.elementRef = elementRef;
        this.viewContainer = viewContainer;
        this.fixed = false;
    }
    /**
     * @param {?} e
     * @return {?}
     */
    onClick(e) {
        e.stopPropagation();
        this.fixed = true;
        this.open();
    }
    /**
     * @param {?} e
     * @return {?}
     */
    onMouseEnter(e) {
        this.open();
    }
    /**
     * @param {?} e
     * @return {?}
     */
    onMouseLeave(e) {
        if (!this.fixed) {
            this.close();
        }
    }
    /**
     * @return {?}
     */
    onWindowClick() {
        this.close();
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.close();
    }
    /**
     * @return {?}
     */
    open() {
        if (this.overlayRef || !this.oavTooltip) {
            return;
        }
        this.overlayRef = this.overlay.create({
            positionStrategy: this.overlay
                .position()
                .flexibleConnectedTo(this.elementRef)
                .withPositions([
                {
                    overlayX: 'start',
                    overlayY: 'top',
                    originX: 'start',
                    originY: 'bottom'
                }
            ])
        });
        this.overlayRef.attach(this.oavTooltip);
    }
    /**
     * @return {?}
     */
    close() {
        this.fixed = false;
        if (this.overlayRef) {
            this.overlayRef.dispose();
            this.overlayRef = null;
        }
    }
}
TooltipDirective.decorators = [
    { type: Directive, args: [{
                selector: '[oavTooltip]'
            },] }
];
/** @nocollapse */
TooltipDirective.ctorParameters = () => [
    { type: Overlay },
    { type: ElementRef },
    { type: ViewContainerRef }
];
TooltipDirective.propDecorators = {
    oavTooltip: [{ type: Input }],
    onClick: [{ type: HostListener, args: ['click', ['$event'],] }],
    onMouseEnter: [{ type: HostListener, args: ['mouseenter', ['$event'],] }],
    onMouseLeave: [{ type: HostListener, args: ['mouseleave', ['$event'],] }],
    onWindowClick: [{ type: HostListener, args: ['window:click',] }]
};
if (false) {
    /** @type {?} */
    TooltipDirective.prototype.oavTooltip;
    /**
     * @type {?}
     * @private
     */
    TooltipDirective.prototype.overlayRef;
    /**
     * @type {?}
     * @private
     */
    TooltipDirective.prototype.fixed;
    /**
     * @type {?}
     * @private
     */
    TooltipDirective.prototype.overlay;
    /**
     * @type {?}
     * @private
     */
    TooltipDirective.prototype.elementRef;
    /**
     * @type {?}
     * @private
     */
    TooltipDirective.prototype.viewContainer;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const ajv = new Ajv({
    unknownFormats: 'ignore'
});
ajv.addFormat('int32', {
    type: 'number',
    validate: (/**
     * @param {?} data
     * @return {?}
     */
    data => !isNaN(data) && data % 1 === 0 && data <= Math.pow(2, 31) && data >= Math.pow(2, 31) * -1)
});
ajv.addFormat('int64', {
    type: 'number',
    validate: (/**
     * @param {?} data
     * @return {?}
     */
    data => !isNaN(data) && data % 1 === 0)
});
ajv.addFormat('float', {
    type: 'number',
    validate: (/**
     * @param {?} data
     * @return {?}
     */
    data => !isNaN(data))
});
ajv.addFormat('double', {
    type: 'number',
    validate: (/**
     * @param {?} data
     * @return {?}
     */
    data => !isNaN(data))
});
ajv.addFormat('byte', {
    type: 'string',
    validate: (/**
     * @param {?} data
     * @return {?}
     */
    data => /^[0-9A-Za-z+/]*=?=?$/.test(data))
});
/**
 * @param {?} schema
 * @return {?}
 */
function getValidationFunction(schema) {
    return ajv.compile(schema);
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CodeInputComponent {
    /**
     * @param {?} cd
     */
    constructor(cd) {
        this.cd = cd;
        this.config = {};
        this.disabled = false;
        this.value = '';
        this.mode = 'text';
        this.minLines = 1;
        this.maxLines = 20;
        this.schema = null;
        this.errors = [];
        this.writing = false;
        this.validate = null;
        this.changes = new Subject();
        this.onChange = (/**
         * @param {?} v
         * @return {?}
         */
        (v) => { });
        this.onTouched = (/**
         * @return {?}
         */
        () => { });
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.changes.pipe(debounceTime(100)).subscribe((/**
         * @return {?}
         */
        () => this.checkValue()));
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.changes.complete();
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (this.schema) {
            try {
                this.validate = getValidationFunction(this.schema);
            }
            catch (e) {
                console.error('Failed to get validation function for schema', this.schema, e);
                this.validate = null;
                this.errors = [e.message];
            }
        }
        else {
            this.validate = null;
            this.errors = [];
        }
        this.updateConfig();
    }
    /**
     * @return {?}
     */
    updateConfig() {
        this.config = {
            minLines: this.minLines,
            maxLines: this.maxLines,
            wrap: true,
            tabSize: 2
        };
        if (this.readonly || this.readonly === '') {
            this.config.readOnly = true;
        }
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    /**
     * @param {?} isDisabled
     * @return {?}
     */
    setDisabledState(isDisabled) {
        this.disabled = isDisabled;
    }
    /**
     * @param {?} obj
     * @return {?}
     */
    writeValue(obj) {
        this.value = obj;
        this.writing = true;
        try {
            this.cd.detectChanges();
        }
        catch (e) {
            console.error('Update Ace error', this.value, e);
        }
        this.writing = false;
        this.changes.next();
    }
    /**
     * @param {?} value
     * @return {?}
     */
    updateValue(value) {
        if (!this.writing) {
            this.value = value;
            this.changes.next();
            this.onChange(value);
        }
    }
    /**
     * @return {?}
     */
    checkValue() {
        if (this.validate && this.mode === 'json') {
            this.errors = [];
            if (!this.value) {
                return;
            }
            try {
                /** @type {?} */
                const text = JSON.parse(this.value);
                if (!this.validate(text)) {
                    this.errors = this.validate.errors.map((/**
                     * @param {?} e
                     * @return {?}
                     */
                    e => `${e.dataPath}: ${e.keyword} ${e.message}`));
                }
            }
            catch (e) {
                this.errors = [e.message];
            }
        }
    }
}
CodeInputComponent.decorators = [
    { type: Component, args: [{
                selector: 'oav-code-input',
                template: "<div>\n  <ace [config]=\"config\" [mode]=\"mode\" [value]=\"value\" (valueChange)=\"updateValue($event)\"></ace>\n</div>\n<div *ngFor=\"let err of errors\" class=\"oav-error\">{{ err }}</div>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef((/**
                         * @return {?}
                         */
                        () => CodeInputComponent)),
                        multi: true
                    }
                ]
            }] }
];
/** @nocollapse */
CodeInputComponent.ctorParameters = () => [
    { type: ChangeDetectorRef }
];
CodeInputComponent.propDecorators = {
    readonly: [{ type: Input }],
    mode: [{ type: Input }],
    minLines: [{ type: Input }],
    maxLines: [{ type: Input }],
    schema: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    CodeInputComponent.prototype.config;
    /** @type {?} */
    CodeInputComponent.prototype.disabled;
    /** @type {?} */
    CodeInputComponent.prototype.value;
    /** @type {?} */
    CodeInputComponent.prototype.readonly;
    /** @type {?} */
    CodeInputComponent.prototype.mode;
    /** @type {?} */
    CodeInputComponent.prototype.minLines;
    /** @type {?} */
    CodeInputComponent.prototype.maxLines;
    /** @type {?} */
    CodeInputComponent.prototype.schema;
    /** @type {?} */
    CodeInputComponent.prototype.errors;
    /**
     * @type {?}
     * @private
     */
    CodeInputComponent.prototype.writing;
    /**
     * @type {?}
     * @private
     */
    CodeInputComponent.prototype.validate;
    /**
     * @type {?}
     * @private
     */
    CodeInputComponent.prototype.changes;
    /**
     * @type {?}
     * @private
     */
    CodeInputComponent.prototype.onChange;
    /**
     * @type {?}
     * @private
     */
    CodeInputComponent.prototype.onTouched;
    /**
     * @type {?}
     * @private
     */
    CodeInputComponent.prototype.cd;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?} op
 * @param {?} pathItem
 * @param {?} fields
 * @return {?}
 */
function getLabel(op, pathItem, fields) {
    for (const field of fields) {
        if (field === 'path') {
            return pathItem.path;
        }
        if (op.operation[field]) {
            return op.operation[field];
        }
    }
    return '';
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class IndexNavEndpointsComponent {
    /**
     * @param {?} oavSettings
     */
    constructor(oavSettings) {
        this.oavSettings = oavSettings;
        if (!this.oavSettings) {
            this.oavSettings = defaultOavSettings;
        }
        this.showHoverLabel = !!this.oavSettings.indexHoverLabel;
    }
    /**
     * @param {?} op
     * @param {?} pathItem
     * @return {?}
     */
    getPrimaryLabel(op, pathItem) {
        return getLabel(op, pathItem, this.oavSettings.indexPrimaryLabel);
    }
    /**
     * @param {?} op
     * @param {?} pathItem
     * @return {?}
     */
    getHoverLabel(op, pathItem) {
        return getLabel(op, pathItem, this.oavSettings.indexHoverLabel);
    }
}
IndexNavEndpointsComponent.decorators = [
    { type: Component, args: [{
                selector: 'oav-index-nav-endpoints',
                template: "<ng-container *ngFor=\"let pathItem of tag.paths\">\n  <ng-container *ngFor=\"let op of pathItem.operations\">\n    <li>\n      <a [routerLink]=\"tag.tag.name + '/' + op.operation.operationId\" routerLinkActive=\"active\">\n        <span class=\"label\">\n          <oav-method class=\"font-size-smaller\" [method]=\"op.method\"></oav-method>\n          <span [class.deprecated]=\"op.operation.deprecated\">{{ getPrimaryLabel(op, pathItem) }}</span>\n        </span>\n        <span class=\"info-overlay\" *ngIf=\"showHoverLabel\">\n          <oav-method class=\"font-size-smaller\" [method]=\"op.method\"></oav-method>\n          <span [class.deprecated]=\"op.operation.deprecated\">{{ getHoverLabel(op, pathItem) }}</span>\n        </span>\n      </a>\n    </li>\n  </ng-container>\n</ng-container>\n",
                changeDetection: ChangeDetectionStrategy.OnPush
            }] }
];
/** @nocollapse */
IndexNavEndpointsComponent.ctorParameters = () => [
    { type: OavSettings, decorators: [{ type: Optional }] }
];
IndexNavEndpointsComponent.propDecorators = {
    tag: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    IndexNavEndpointsComponent.prototype.tag;
    /** @type {?} */
    IndexNavEndpointsComponent.prototype.showHoverLabel;
    /**
     * @type {?}
     * @private
     */
    IndexNavEndpointsComponent.prototype.oavSettings;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class OverviewViewEndpointsComponent {
    /**
     * @param {?} oavSettings
     */
    constructor(oavSettings) {
        this.oavSettings = oavSettings;
        if (!this.oavSettings) {
            this.oavSettings = defaultOavSettings;
        }
    }
    /**
     * @param {?} op
     * @param {?} pathItem
     * @return {?}
     */
    getLabel(op, pathItem) {
        return getLabel(op, pathItem, this.oavSettings.overviewPrimaryLabel);
    }
}
OverviewViewEndpointsComponent.decorators = [
    { type: Component, args: [{
                selector: 'oav-overview-view-endpoints',
                template: "<ng-container *ngFor=\"let pathItem of tag.paths\">\n  <ng-container *ngFor=\"let op of pathItem.operations\">\n    <h3>\n      <oav-method [method]=\"op.method\"></oav-method>\n      <oav-auth-status [operation]=\"op.operation\"></oav-auth-status>\n      <a [routerLink]=\"tag.tag.name + '/' + op.operation.operationId\" routerLinkActive=\"active\" class=\"oav-header-link ml-1\">\n        <span class=\"oav-path\" [class.deprecated]=\"op.operation.deprecated\">{{ pathItem.path }}</span>\n        <span class=\"oav-summary font-size-smaller ml-2\">{{ getLabel(op, pathItem) }}</span>\n      </a>\n    </h3>\n  </ng-container>\n</ng-container>\n",
                changeDetection: ChangeDetectionStrategy.OnPush
            }] }
];
/** @nocollapse */
OverviewViewEndpointsComponent.ctorParameters = () => [
    { type: OavSettings, decorators: [{ type: Optional }] }
];
OverviewViewEndpointsComponent.propDecorators = {
    tag: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    OverviewViewEndpointsComponent.prototype.tag;
    /**
     * @type {?}
     * @private
     */
    OverviewViewEndpointsComponent.prototype.oavSettings;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @record
 */
function ModelInfo() { }
if (false) {
    /** @type {?} */
    ModelInfo.prototype.name;
    /** @type {?} */
    ModelInfo.prototype.type;
    /** @type {?} */
    ModelInfo.prototype.schema;
    /** @type {?|undefined} */
    ModelInfo.prototype.properties;
    /** @type {?|undefined} */
    ModelInfo.prototype.open;
}
/**
 * @record
 */
function PropertyInfo() { }
if (false) {
    /** @type {?} */
    PropertyInfo.prototype.required;
    /** @type {?|undefined} */
    PropertyInfo.prototype.modelName;
}
/**
 * @record
 */
function SchemaInfo() { }
if (false) {
    /** @type {?} */
    SchemaInfo.prototype.additionalModels;
}
/**
 * @param {?} defaultName
 * @param {?} schema
 * @return {?}
 */
function identifySchemaInfo(defaultName, schema) {
    /** @type {?} */
    const additionalModels = [];
    /** @type {?} */
    const modelInfo = identifyModelInfo(defaultName, readSchema(schema), additionalModels);
    return Object.assign({}, modelInfo, { additionalModels });
}
/**
 * @param {?} defaultName
 * @param {?} schema
 * @param {?} additionalModels
 * @return {?}
 */
function identifyModelInfo(defaultName, schema, additionalModels) {
    /** @type {?} */
    const info = {
        name: defaultName,
        type: readType(schema),
        schema: readSchema(schema),
        properties: null
    };
    /** @type {?} */
    const modelName = readModelName(info.schema);
    if (modelName) {
        info.name = modelName;
    }
    if (info.type === 'object' || info.type === 'array') {
        info.properties = readProperties(info.schema, additionalModels);
    }
    return info;
}
/**
 * @param {?} schema
 * @param {?} additionalModels
 * @return {?}
 */
function readProperties(schema, additionalModels) {
    /** @type {?} */
    const properties = [];
    if (schema.properties) {
        properties.push(...readPropertiesOfObject(schema));
    }
    if (schema.additionalProperties) {
        properties.push(readProperty('...', (/** @type {?} */ (schema.additionalProperties))));
    }
    if (Array.isArray(schema.items)) {
        for (let i = 0; i < schema.items.length; i++) {
            properties.push(readProperty(i.toString(), schema.items[i]));
        }
    }
    else if (schema.items) {
        properties.push(readProperty('', schema.items));
    }
    if (schema.additionalItems) {
        properties.push(readProperty('...', schema.additionalItems));
    }
    for (const prop of properties) {
        /** @type {?} */
        const identifiedModel = updateModelsFromProperty(prop.name + 'Object', prop.schema, additionalModels);
        prop.modelName = identifiedModel;
        if (!identifiedModel && (prop.type === 'object' || prop.type === 'array')) {
            prop.properties = readProperties(prop.schema, additionalModels);
        }
    }
    return properties;
}
/**
 * @param {?} defaultName
 * @param {?} schema
 * @param {?} additionalModels
 * @return {?}
 */
function updateModelsFromProperty(defaultName, schema, additionalModels) {
    if (schema.$ref) {
        // Unknown schema, probably did not resolved because of recursive structure
        /** @type {?} */
        const parts = schema.$ref.split('/');
        return parts.pop();
    }
    /** @type {?} */
    const refName = readModelName(schema);
    if (schema.type === 'object') {
        if (!schema.properties && !schema.additionalProperties) {
            return null;
        }
        /** @type {?} */
        const modelName = refName || defaultName;
        if (!additionalModels.find((/**
         * @param {?} m
         * @return {?}
         */
        m => m.name === modelName))) {
            /** @type {?} */
            const designatedIndex = additionalModels.length;
            /** @type {?} */
            const modelInfo = identifyModelInfo(modelName, schema, additionalModels);
            additionalModels.splice(designatedIndex, 0, modelInfo);
        }
        return modelName;
    }
    return null;
}
/**
 * @param {?} schema
 * @return {?}
 */
function readSchema(schema) {
    if (typeof schema === 'boolean') {
        return { type: 'boolean' };
    }
    return schema;
}
/**
 * @param {?} schema
 * @return {?}
 */
function readType(schema) {
    if (schema && typeof schema === 'object' && schema.type) {
        return Array.isArray(schema.type) ? schema.type[0] : schema.type;
    }
    if (typeof schema === 'boolean') {
        return 'boolean';
    }
    return null;
}
/**
 * @param {?} schema
 * @return {?}
 */
function readModelName(schema) {
    if (((/** @type {?} */ (schema))).$$ref) {
        /** @type {?} */
        const refParts = ((/** @type {?} */ (schema))).$$ref.split('/');
        return refParts.pop();
    }
    return undefined;
}
/**
 * @param {?} schema
 * @return {?}
 */
function readPropertiesOfObject(schema) {
    if (schema.properties) {
        /** @type {?} */
        const requiredFields = schema.required || [];
        return Object.keys(schema.properties).map((/**
         * @param {?} name
         * @return {?}
         */
        name => readProperty(name, schema.properties[name], requiredFields.includes(name))));
    }
    return [];
}
/**
 * @param {?} name
 * @param {?} schema
 * @param {?=} required
 * @return {?}
 */
function readProperty(name, schema, required = false) {
    /** @type {?} */
    const s = readSchema(schema);
    /** @type {?} */
    const type = Array.isArray(s.type) ? s.type[0] : s.type;
    return { name, type, schema: s, required };
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class JsonSchemaComponent {
    constructor() { }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        /** @type {?} */
        const info = identifySchemaInfo(this.defaultModelName, this.schema);
        info.open = true;
        this.models = [info, ...info.additionalModels];
    }
    /**
     * @param {?} type
     * @return {?}
     */
    isPrimitive(type) {
        return type === 'integer' || type === 'number' || type === 'string' || type === 'boolean' || type === 'null' || type === 'any';
    }
    /**
     * @param {?} model
     * @return {?}
     */
    toggleModel(model) {
        if (model.open) {
            model.open = false;
        }
        else {
            model.open = true;
        }
    }
    /**
     * @param {?} name
     * @return {?}
     */
    toggleModelByName(name) {
        /** @type {?} */
        const modelInfo = this.models.find((/**
         * @param {?} m
         * @return {?}
         */
        m => m.name === name));
        if (modelInfo) {
            this.toggleModel(modelInfo);
        }
    }
}
JsonSchemaComponent.decorators = [
    { type: Component, args: [{
                selector: 'oav-json-schema',
                template: "<div *ngFor=\"let model of models\">\n  <ng-container *ngIf=\"model.type === 'object'\">\n    <div *ngIf=\"!model.open\" class=\"oav-json-schema-code\">\n      <a href=\"javascript:\" (click)=\"toggleModel(model)\" class=\"oav-json-schema-toggle\">\n        {{ model.name }}\n      </a>\n      {{ '{' }} <a href=\"javascript:\" (click)=\"toggleModel(model)\" class=\"oav-json-schema-toggle\">...</a> {{ '}' }}\n    </div>\n    <ng-container *ngIf=\"model.open\">\n      <div class=\"oav-json-schema-code\">\n        <a href=\"javascript:\" (click)=\"toggleModel(model)\" class=\"oav-json-schema-toggle\">\n          {{ model.name }}\n        </a>\n        {{ '{' }}\n      </div>\n      <div class=\"ml-3 json-schema-property\" *ngFor=\"let prop of model.properties\">\n        <div class=\"oav-json-schema-name oav-json-schema-code\" [class.oav-json-schema-required]=\"prop.required\">{{ prop.name }}:</div>\n        <div class=\"oav-json-schema-type\" *ngIf=\"!prop.modelName\">\n          <ng-container *ngTemplateOutlet=\"typeOutput; context: prop\"></ng-container>\n        </div>\n        <div class=\"oav-json-schema-type oav-json-schema-code\" *ngIf=\"prop.modelName\">\n          <a href=\"javascript:\" (click)=\"toggleModelByName(prop.modelName)\" class=\"oav-json-schema-toggle\">\n            {{ prop.modelName }}\n          </a>\n        </div>\n        <div class=\"oav-json-schema-description\">\n          <markdown *ngIf=\"prop.schema?.description\" [data]=\"prop.schema?.description\"></markdown>\n        </div>\n      </div>\n      <div class=\"oav-json-schema-code\">{{ '}' }}</div>\n    </ng-container>\n  </ng-container>\n  <ng-container *ngIf=\"model.type === 'array'\">\n    <ng-container *ngTemplateOutlet=\"arrayOutput; context: model\"></ng-container>\n  </ng-container>\n  <ng-container *ngIf=\"isPrimitive(model.type)\">\n    <ng-container *ngTemplateOutlet=\"typeOutput; context: model\"></ng-container>\n  </ng-container>\n</div>\n\n<ng-template #arrayOutput let-properties=\"properties\">\n  <ng-container *ngIf=\"properties?.length === 1 && properties[0] as prop; else multipleProps\">\n    <div class=\"oav-json-schema-code\" *ngIf=\"prop.modelName\">\n      [\n      <a href=\"javascript:\" (click)=\"toggleModelByName(prop.modelName)\" class=\"oav-json-schema-toggle\">\n        {{ prop.modelName }}\n      </a>\n      ]\n    </div>\n    <div class=\"oav-json-schema-code\" *ngIf=\"!prop.modelName\">\n      [ <span class=\"oav-json-schema-type-name\">{{ prop.type }}</span> ]\n    </div>\n  </ng-container>\n  <ng-template #multipleProps>\n    <div class=\"oav-json-schema-code\">[</div>\n    <div class=\"ml-3 json-schema-property\" *ngFor=\"let prop of properties\">\n      <div class=\"oav-json-schema-name oav-json-schema-code\" *ngIf=\"prop.name\">{{ prop.name }}:</div>\n      <div class=\"oav-json-schema-type\" *ngIf=\"!prop.modelName\">\n        <ng-container *ngTemplateOutlet=\"typeOutput; context: prop\"></ng-container>\n      </div>\n      <div class=\"oav-json-schema-type oav-json-schema-code\" *ngIf=\"prop.modelName\">\n        <a href=\"javascript:\" (click)=\"toggleModelByName(prop.modelName)\" class=\"oav-json-schema-toggle\">\n          {{ prop.modelName }}\n        </a>\n      </div>\n      <div class=\"oav-json-schema-description\">\n        <markdown *ngIf=\"prop.schema?.description\" [data]=\"prop.schema?.description\"></markdown>\n      </div>\n    </div>\n    <div class=\"oav-json-schema-code\">]</div>\n  </ng-template>\n</ng-template>\n\n<ng-template #typeOutput let-type=\"type\" let-schema=\"schema\" let-properties=\"properties\">\n  <ng-container *ngIf=\"type === 'array'\">\n    <ng-container *ngTemplateOutlet=\"arrayOutput; context: { properties: properties }\"></ng-container>\n  </ng-container>\n  <ng-container *ngIf=\"type !== 'array'\">\n    <span class=\"oav-json-schema-type-name oav-json-schema-code\">\n      {{ type }}\n    </span>\n    <span class=\"oav-json-schema-format oav-json-schema-code\" *ngIf=\"schema?.format\">({{ schema?.format }})</span>\n    <span *ngIf=\"schema?.enum?.length\" class=\"oav-json-schema-code\">({{ schema.enum.join(' | ') }})</span>\n  </ng-container>\n</ng-template>\n",
                changeDetection: ChangeDetectionStrategy.OnPush
            }] }
];
/** @nocollapse */
JsonSchemaComponent.ctorParameters = () => [];
JsonSchemaComponent.propDecorators = {
    schema: [{ type: Input }],
    defaultModelName: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    JsonSchemaComponent.prototype.schema;
    /** @type {?} */
    JsonSchemaComponent.prototype.defaultModelName;
    /** @type {?} */
    JsonSchemaComponent.prototype.models;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class OpenapiViewerModule {
}
OpenapiViewerModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    OpenapiViewerComponent,
                    OperationComponent,
                    ParameterComponent,
                    ResponsesComponent,
                    MultiItemsInputComponent,
                    RequestsListComponent,
                    RequestViewComponent,
                    OpenableDirective,
                    MethodHeaderComponent,
                    MethodComponent,
                    IndexNavComponent,
                    OperationViewComponent,
                    OverviewViewComponent,
                    TabNavComponent,
                    TabNavItemDirective,
                    TemplateOutletComponent,
                    RequestBodyInputComponent,
                    AuthenticationViewComponent,
                    AuthCredentialsComponent,
                    AuthStatusComponent,
                    TooltipComponent,
                    TooltipDirective,
                    CodeInputComponent,
                    IndexNavEndpointsComponent,
                    OverviewViewEndpointsComponent,
                    ModelsViewComponent,
                    JsonSchemaComponent
                ],
                imports: [
                    CommonModule,
                    OpenapiViewerRoutingModule,
                    FormsModule,
                    ReactiveFormsModule,
                    HttpClientModule,
                    MarkdownModule.forRoot(),
                    PortalModule,
                    OverlayModule,
                    AceModule
                ],
                exports: [OpenapiViewerComponent, OperationComponent]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @record
 */
function PathItem() { }
if (false) {
    /** @type {?} */
    PathItem.prototype.path;
    /** @type {?} */
    PathItem.prototype.operations;
    /** @type {?|undefined} */
    PathItem.prototype.summary;
    /** @type {?|undefined} */
    PathItem.prototype.description;
    /** @type {?|undefined} */
    PathItem.prototype.servers;
    /** @type {?|undefined} */
    PathItem.prototype.parameters;
}
/**
 * @record
 */
function TagIndex() { }
if (false) {
    /** @type {?} */
    TagIndex.prototype.tag;
    /** @type {?} */
    TagIndex.prototype.paths;
}
/**
 * @record
 */
function OperationsItem() { }
if (false) {
    /** @type {?} */
    OperationsItem.prototype.method;
    /** @type {?} */
    OperationsItem.prototype.operation;
    /** @type {?} */
    OperationsItem.prototype.parameters;
    /** @type {?} */
    OperationsItem.prototype.responseTypes;
}
/**
 * @record
 */
function BasicAuthCredentials() { }
if (false) {
    /** @type {?} */
    BasicAuthCredentials.prototype.username;
    /** @type {?} */
    BasicAuthCredentials.prototype.password;
}
/**
 * @record
 */
function OAuthToken() { }
if (false) {
    /** @type {?} */
    OAuthToken.prototype.access_token;
    /** @type {?} */
    OAuthToken.prototype.token_type;
    /** @type {?|undefined} */
    OAuthToken.prototype.refresh_token;
    /** @type {?|undefined} */
    OAuthToken.prototype.id_token;
    /** @type {?|undefined} */
    OAuthToken.prototype.expires_at;
}
/**
 * @record
 */
function OAuthCredentials() { }
if (false) {
    /** @type {?} */
    OAuthCredentials.prototype.token;
    /** @type {?} */
    OAuthCredentials.prototype.flow;
    /** @type {?} */
    OAuthCredentials.prototype.clientId;
    /** @type {?|undefined} */
    OAuthCredentials.prototype.clientSecret;
    /** @type {?|undefined} */
    OAuthCredentials.prototype.username;
    /** @type {?|undefined} */
    OAuthCredentials.prototype.password;
    /** @type {?} */
    OAuthCredentials.prototype.scopes;
    /** @type {?} */
    OAuthCredentials.prototype.nonce;
    /** @type {?|undefined} */
    OAuthCredentials.prototype.redirectUri;
}
/**
 * @record
 */
function SecuritySchemeItem() { }
if (false) {
    /** @type {?} */
    SecuritySchemeItem.prototype.name;
    /** @type {?} */
    SecuritySchemeItem.prototype.securityScheme;
    /** @type {?} */
    SecuritySchemeItem.prototype.authenticated;
    /** @type {?} */
    SecuritySchemeItem.prototype.remember;
    /** @type {?} */
    SecuritySchemeItem.prototype.credentials;
}
/**
 * @record
 */
function SecurityRequirementStatus() { }
if (false) {
    /** @type {?} */
    SecurityRequirementStatus.prototype.name;
    /** @type {?} */
    SecurityRequirementStatus.prototype.securityScheme;
    /** @type {?} */
    SecurityRequirementStatus.prototype.authenticated;
    /** @type {?|undefined} */
    SecurityRequirementStatus.prototype.scopes;
}
/**
 * @record
 */
function ResponseItem() { }
if (false) {
    /** @type {?} */
    ResponseItem.prototype.status;
}
/**
 * @record
 */
function OavRequest() { }
if (false) {
    /** @type {?} */
    OavRequest.prototype.id;
    /** @type {?} */
    OavRequest.prototype.pathItem;
    /** @type {?} */
    OavRequest.prototype.operationsItem;
    /** @type {?} */
    OavRequest.prototype.request;
    /** @type {?} */
    OavRequest.prototype.running;
    /** @type {?} */
    OavRequest.prototype.canceled;
    /** @type {?} */
    OavRequest.prototype.sentBytes;
    /** @type {?} */
    OavRequest.prototype.receivedBytes;
    /** @type {?} */
    OavRequest.prototype.status;
    /** @type {?} */
    OavRequest.prototype.statusText;
    /** @type {?|undefined} */
    OavRequest.prototype.error;
    /** @type {?} */
    OavRequest.prototype.startTs;
    /** @type {?} */
    OavRequest.prototype.endTs;
    /** @type {?} */
    OavRequest.prototype.httpEvents;
    /** @type {?} */
    OavRequest.prototype.cancel;
}
/**
 * @record
 */
function SwaggerRequest() { }
if (false) {
    /** @type {?} */
    SwaggerRequest.prototype.method;
    /** @type {?} */
    SwaggerRequest.prototype.url;
    /** @type {?} */
    SwaggerRequest.prototype.headers;
    /** @type {?} */
    SwaggerRequest.prototype.cookies;
    /** @type {?|undefined} */
    SwaggerRequest.prototype.body;
}
/**
 * @record
 */
function ScopesInfo() { }
if (false) {
    /** @type {?} */
    ScopesInfo.prototype.scope;
    /** @type {?} */
    ScopesInfo.prototype.description;
}
/**
 * @record
 */
function FlowInfo() { }
if (false) {
    /** @type {?} */
    FlowInfo.prototype.flow;
    /** @type {?} */
    FlowInfo.prototype.scopes;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { OavSettings, OpenapiAuthService, OpenapiViewerComponent, OpenapiViewerModule, OpenapiViewerService, buildAuthorizationUrl, defaultOavSettings, getAuthorizationBaseUrl, getTagsOfPath, getTokenBaseUrl, identifyFlows, OperationComponent as ɵa, ParameterComponent as ɵb, OpenapiViewerRoutingModule as ɵba, ResponsesComponent as ɵc, MultiItemsInputComponent as ɵd, RequestsListComponent as ɵe, RequestViewComponent as ɵf, OpenableDirective as ɵg, MethodHeaderComponent as ɵh, MethodComponent as ɵi, IndexNavComponent as ɵj, OperationViewComponent as ɵk, OverviewViewComponent as ɵl, TabNavComponent as ɵm, TabNavItemDirective as ɵn, TemplateOutletComponent as ɵo, RequestBodyInputComponent as ɵp, AuthenticationViewComponent as ɵq, AuthCredentialsComponent as ɵr, AuthStatusComponent as ɵs, TooltipComponent as ɵt, TooltipDirective as ɵu, CodeInputComponent as ɵv, IndexNavEndpointsComponent as ɵw, OverviewViewEndpointsComponent as ɵx, ModelsViewComponent as ɵy, JsonSchemaComponent as ɵz };
//# sourceMappingURL=dhcode-openapi-viewer.js.map
