import { NgZone } from '@angular/core';
import { BehaviorSubject, Observable, Subject } from 'rxjs';
import { AuthStatus, FlowInfo, OAuthCredentials, OAuthFlow, SecurityCredentials, SecurityRequirementStatus, SecuritySchemeItem } from '../models/openapi-viewer.model';
import { OpenAPIObject, SecurityRequirementObject, SecuritySchemeObject } from 'openapi3-ts';
import { HttpClient } from '@angular/common/http';
export declare class OpenapiAuthService {
    private zone;
    private http;
    readonly securitySchemes: BehaviorSubject<SecuritySchemeItem[]>;
    /**
     * Emits with the scheme name when its credentials have been updated
     */
    readonly updatedCredentials: Subject<string>;
    private globalRequirements;
    currentWindow: Window;
    constructor(zone: NgZone, http: HttpClient);
    identifySchemes(spec: OpenAPIObject): void;
    getAllRequirements(requirement?: SecurityRequirementObject[]): SecurityRequirementObject[];
    getRequiredSchemes(requirement?: SecurityRequirementObject[]): SecurityRequirementStatus[][];
    getAuthStatus(requirements: SecurityRequirementObject[]): AuthStatus;
    hasAnyAuthorization(requirements: SecurityRequirementObject[]): boolean;
    hasRequirement(reqName: any, scopes: string[]): boolean;
    getSchema(name: string): SecuritySchemeItem;
    updateCredentials(name: string, credentials: SecurityCredentials, authenticated: boolean, remember?: boolean): void;
    runOAuthAuthorization(name: string, credentials: OAuthCredentials): Observable<boolean>;
    /**
     * @param params
     * eg. access_token=8c665834-6a43-4704-81d3-42d311a1ab91&token_type=bearer&expires_in=20&scope=read%20write:pets%20write%20read:pets
     */
    handleOAuthCallback(params: Record<string, string>): Promise<void>;
    private updateByImplicitFlow;
    private updateByCodeFlow;
    /**
     * Updates the provided credentials with a token
     */
    private requestToken;
    removeToken(name: string): void;
}
export declare function identifyFlows(scheme: SecuritySchemeObject): FlowInfo[];
export declare function getAuthorizationBaseUrl(scheme: SecuritySchemeObject, flow: OAuthFlow | 'application' | 'accessCode'): any;
export declare function getTokenBaseUrl(scheme: SecuritySchemeObject, flow: OAuthFlow | 'application' | 'accessCode'): any;
export declare function buildAuthorizationUrl(scheme: SecuritySchemeObject, credentials: OAuthCredentials, state?: string): string;
